@@ -123,7 +123,13 @@ static struct my_option my_long_options[] =
   {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
-  {"port", 'P', "Port number to use for connection.", (gptr*) &opt_mysql_port,
+  {"port", 'P', "Port number to use for connection or 0 for default to, in "
+   "order of preference, my.cnf, $MYSQL_TCP_PORT, "
+#if MYSQL_PORT_DEFAULT == 0
+   "/etc/services, "
+#endif
+   "built-in default (" STRINGIFY_ARG(MYSQL_PORT) ").",
+   (gptr*) &opt_mysql_port,
    (gptr*) &opt_mysql_port, 0, GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0,
    0},
   {"protocol", OPT_MYSQL_PROTOCOL, "The protocol of connection (tcp,socket,pipe,memory).",
@@ -180,6 +186,7 @@ static void dbDisconnect(char *host);
 static void DBerror(MYSQL *mysql, const char *when);
 static void safe_exit(int error);
 static void print_result();
+static uint fixed_name_length(const char *name);
 static char *fix_table_name(char *dest, char *src);
 int what_to_do = 0;
 
@@ -403,14 +410,14 @@ static int process_selected_tables(char *db, char **table_names, int tables)
   {
     /* 
       We need table list in form `a`, `b`, `c`
-      that's why we need 4 more chars added to to each table name
+      that's why we need 2 more chars added to to each table name
       space is for more readable output in logs and in case of error
     */	  
     char *table_names_comma_sep, *end;
     int i, tot_length = 0;
 
     for (i = 0; i < tables; i++)
-      tot_length += strlen(*(table_names + i)) + 4;
+      tot_length+= fixed_name_length(*(table_names + i)) + 2;
 
     if (!(table_names_comma_sep = (char *)
 	  my_malloc((sizeof(char) * tot_length) + 4, MYF(MY_WME))))
@@ -428,23 +435,46 @@ static int process_selected_tables(char *db, char **table_names, int tables)
   }
   else
     for (; tables > 0; tables--, table_names++)
-      handle_request_for_tables(*table_names, strlen(*table_names));
+      handle_request_for_tables(*table_names, fixed_name_length(*table_names));
   return 0;
 } /* process_selected_tables */
 
 
+static uint fixed_name_length(const char *name)
+{
+  const char *p;
+  uint extra_length= 2;  /* count the first/last backticks */
+  
+  for (p= name; *p; p++)
+  {
+    if (*p == '`')
+      extra_length++;
+    else if (*p == '.')
+      extra_length+= 2;
+  }
+  return (p - name) + extra_length;
+}
+
+
 static char *fix_table_name(char *dest, char *src)
 {
-  char *db_sep;
-
   *dest++= '`';
-  if ((db_sep= strchr(src, '.')))
+  for (; *src; src++)
   {
-    dest= strmake(dest, src, (uint) (db_sep - src));
-    dest= strmov(dest, "`.`");
-    src= db_sep + 1;
+    switch (*src) {
+    case '.':            /* add backticks around '.' */
+      *dest++= '`';
+      *dest++= '.';
+      *dest++= '`';
+      break;
+    case '`':            /* escape backtick character */
+      *dest++= '`';
+      /* fall through */
+    default:
+      *dest++= *src;
+    }
   }
-  dest= strxmov(dest, src, "`", NullS);
+  *dest++= '`';
   return dest;
 }
 
@@ -468,7 +498,7 @@ static int process_all_tables_in_db(char *database)
   {
     /*
       We need table list in form `a`, `b`, `c`
-      that's why we need 4 more chars added to to each table name
+      that's why we need 2 more chars added to to each table name
       space is for more readable output in logs and in case of error
      */
 
@@ -476,7 +506,7 @@ static int process_all_tables_in_db(char *database)
     uint tot_length = 0;
 
     while ((row = mysql_fetch_row(res)))
-      tot_length += strlen(row[0]) + 4;
+      tot_length+= fixed_name_length(row[0]) + 2;
     mysql_data_seek(res, 0);
 
     if (!(tables=(char *) my_malloc(sizeof(char)*tot_length+4, MYF(MY_WME))))
@@ -500,12 +530,14 @@ static int process_all_tables_in_db(char *database)
   else
   {
     while ((row = mysql_fetch_row(res)))
-    {
-      if ((num_columns == 2) && (strcmp(row[1], "VIEW") == 0))
-        continue;
-
-      handle_request_for_tables(row[0], strlen(row[0]));
-    }
+      /* Skip tables with an engine of NULL (probably a view). */
+      if (row[1])
+      {
+        if ((num_columns == 2) && (strcmp(row[1], "VIEW") == 0))
+          continue;
+        
+        handle_request_for_tables(row[0], fixed_name_length(row[0]));
+      }
   }
   mysql_free_result(res);
   return 0;
@@ -738,7 +770,7 @@ int main(int argc, char **argv)
     for (i = 0; i < tables4repair.elements ; i++)
     {
       char *name= (char*) dynamic_array_ptr(&tables4repair, i);
-      handle_request_for_tables(name, strlen(name));
+      handle_request_for_tables(name, fixed_name_length(name));
     }
   }
  end:
