@@ -57,8 +57,7 @@ public:
     word32  GetBitCountLo() const { return  loLen_ << 3; }
     word32  GetBitCountHi() const { return (loLen_ >> (8*sizeof(loLen_) - 3)) +
                                            (hiLen_ << 3); } 
-
-    enum { MaxDigestSz = 5, MaxBufferSz = 64 };
+    enum { MaxDigestSz = 8, MaxBufferSz = 64 };
 protected:
     typedef word32 HashLengthType;
     word32          buffLen_;   // in bytes
@@ -73,6 +72,38 @@ protected:
 };
 
 
+#ifdef WORD64_AVAILABLE
+
+// 64-bit HASH with Transform
+class HASH64withTransform : public HASH {
+public:
+    HASH64withTransform(word32 digSz, word32 buffSz);
+    virtual ~HASH64withTransform() {}
+    virtual ByteOrder getByteOrder()  const = 0;
+    virtual word32    getPadSize()    const = 0;
+
+    virtual void Update(const byte*, word32);
+    virtual void Final(byte*);
+
+    word32  GetBitCountLo() const { return  loLen_ << 3; }
+    word32  GetBitCountHi() const { return (loLen_ >> (8*sizeof(loLen_) - 3)) +
+                                           (hiLen_ << 3); } 
+    enum { MaxDigestSz = 8, MaxBufferSz = 128 };
+protected:
+    typedef word32 HashLengthType;
+    word32          buffLen_;   // in bytes
+    HashLengthType  loLen_;     // length in bytes
+    HashLengthType  hiLen_;     // length in bytes
+    word64          digest_[MaxDigestSz];
+    word64          buffer_[MaxBufferSz / sizeof(word64)];
+
+    virtual void Transform() = 0;
+
+    void AddLength(word32);
+};
+
+#endif // WORD64_AVAILABLE
+
 
 } // namespace
 
