@@ -29,7 +29,9 @@ UNIV_INLINE
 ibool
 page_cur_try_search_shortcut(
 /*=========================*/
+				/* out: TRUE on success */
 	page_t*		page,	/* in: index page */
+	dict_index_t*	index,	/* in: record descriptor */
 	dtuple_t*	tuple,	/* in: data tuple */
 	ulint*		iup_matched_fields,
 				/* in/out: already matched fields in upper
@@ -45,7 +47,6 @@ page_cur_try_search_shortcut(
 				not yet completely matched */
 	page_cur_t*	cursor) /* out: page cursor */ 
 {
-	int	cmp;
 	rec_t*	rec;
 	rec_t*	next_rec;
 	ulint	low_match;
@@ -55,9 +56,17 @@ page_cur_try_search_shortcut(
 #ifdef UNIV_SEARCH_DEBUG
 	page_cur_t cursor2;
 #endif
+	ibool		success		= FALSE;
+	mem_heap_t*	heap		= NULL;
+	ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+	ulint*		offsets		= offsets_;
+	*offsets_ = (sizeof offsets_) / sizeof *offsets_;
+
 	ut_ad(dtuple_check_typed(tuple));
 
 	rec = page_header_get_ptr(page, PAGE_LAST_INSERT);
+	offsets = rec_get_offsets(rec, index, offsets,
+				dtuple_get_n_fields(tuple), &heap);
 
 	ut_ad(rec);
 	ut_ad(page_rec_is_user_rec(rec));
@@ -69,26 +78,24 @@ page_cur_try_search_shortcut(
 	up_match = low_match;
 	up_bytes = low_bytes;
 
-	cmp = page_cmp_dtuple_rec_with_match(tuple, rec, &low_match,
-								&low_bytes);
-	if (cmp == -1) {
-
-		return(FALSE);
+	if (page_cmp_dtuple_rec_with_match(tuple, rec, offsets,
+						&low_match, &low_bytes) < 0) {
+		goto exit_func;
 	}
 
 	next_rec = page_rec_get_next(rec);
+	offsets = rec_get_offsets(next_rec, index, offsets,
+				dtuple_get_n_fields(tuple), &heap);
 
-	cmp = page_cmp_dtuple_rec_with_match(tuple, next_rec, &up_match,
-								&up_bytes);
-	if (cmp != -1) {
-
-		return(FALSE);
+	if (page_cmp_dtuple_rec_with_match(tuple, next_rec, offsets,
+						&up_match, &up_bytes) >= 0) {
+		goto exit_func;
 	}
 
 	cursor->rec = rec;
 
 #ifdef UNIV_SEARCH_DEBUG
-	page_cur_search_with_match(page, tuple, PAGE_CUR_DBG,
+	page_cur_search_with_match(page, index, tuple, PAGE_CUR_DBG,
 	    					iup_matched_fields,
 	    					iup_matched_bytes,
 	    					ilow_matched_fields,
@@ -105,7 +112,7 @@ page_cur_try_search_shortcut(
 	ut_a(*ilow_matched_fields == low_match);
 	ut_a(*ilow_matched_bytes == low_bytes);
 #endif
-	if (next_rec != page_get_supremum_rec(page)) {
+	if (!page_rec_is_supremum(next_rec)) {
 
 		*iup_matched_fields = up_match;
 		*iup_matched_bytes = up_bytes;
@@ -117,11 +124,17 @@ page_cur_try_search_shortcut(
 #ifdef UNIV_SEARCH_PERF_STAT
 	page_cur_short_succ++;
 #endif
-	return(TRUE);
+	success = TRUE;
+exit_func:
+	if (UNIV_LIKELY_NULL(heap)) {
+		mem_heap_free(heap);
+	}
+	return(success);
 }
 
 #endif
 
+#ifdef PAGE_CUR_LE_OR_EXTENDS
 /********************************************************************
 Checks if the nth field in a record is a character type field which extends
 the nth field in tuple, i.e., the field is longer or equal in length and has
@@ -130,22 +143,24 @@ static
 ibool
 page_cur_rec_field_extends(
 /*=======================*/
-			   /* out: TRUE if rec field extends tuple
-			   field */
-	dtuple_t* tuple,   /* in: data tuple */
-	rec_t*    rec,     /* in: record */
-        ulint     n)       /* in: compare nth field */
+				/* out: TRUE if rec field
+				extends tuple field */
+	dtuple_t*	tuple,	/* in: data tuple */
+	rec_t*		rec,	/* in: record */
+	const ulint*	offsets,/* in: array returned by rec_get_offsets() */
+	ulint		n)	/* in: compare nth field */
 {
         dtype_t* type;
         dfield_t* dfield;
         byte*     rec_f;
         ulint     rec_f_len;
 
+	ut_ad(rec_offs_validate(rec, NULL, offsets));
         dfield = dtuple_get_nth_field(tuple, n);
 
         type = dfield_get_type(dfield);
 
-        rec_f = rec_get_nth_field(rec, n, &rec_f_len);
+	rec_f = rec_get_nth_field(rec, offsets, n, &rec_f_len);
 
         if (type->mtype == DATA_VARCHAR
            || type->mtype == DATA_CHAR
@@ -168,6 +183,7 @@ page_cur_rec_field_extends(
 
         return(FALSE);
 }
+#endif /* PAGE_CUR_LE_OR_EXTENDS */
 
 /********************************************************************
 Searches the right position for a page cursor. */
@@ -176,6 +192,7 @@ void
 page_cur_search_with_match(
 /*=======================*/
 	page_t*		page,	/* in: index page */
+	dict_index_t*	index,	/* in: record descriptor */
 	dtuple_t*	tuple,	/* in: data tuple */
 	ulint		mode,	/* in: PAGE_CUR_L, PAGE_CUR_LE, PAGE_CUR_G,
 				or PAGE_CUR_GE */
@@ -212,14 +229,26 @@ page_cur_search_with_match(
 	ulint	dbg_matched_fields;
 	ulint	dbg_matched_bytes;
 #endif
+	mem_heap_t*	heap		= NULL;
+	ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+	ulint*		offsets		= offsets_;
+	*offsets_ = (sizeof offsets_) / sizeof *offsets_;
+
 	ut_ad(page && tuple && iup_matched_fields && iup_matched_bytes
 	      && ilow_matched_fields && ilow_matched_bytes && cursor);
 	ut_ad(dtuple_validate(tuple));
 	ut_ad(dtuple_check_typed(tuple));
+#ifdef UNIV_DEBUG
+# ifdef PAGE_CUR_DBG
+	if (mode != PAGE_CUR_DBG)
+# endif /* PAGE_CUR_DBG */
+# ifdef PAGE_CUR_LE_OR_EXTENDS
+		if (mode != PAGE_CUR_LE_OR_EXTENDS)
+# endif /* PAGE_CUR_LE_OR_EXTENDS */
 	ut_ad((mode == PAGE_CUR_L) || (mode == PAGE_CUR_LE)
-	      || (mode == PAGE_CUR_G) || (mode == PAGE_CUR_GE)
-	      || (mode == PAGE_CUR_LE_OR_EXTENDS) || (mode == PAGE_CUR_DBG));
-	      
+	      || (mode == PAGE_CUR_G) || (mode == PAGE_CUR_GE));
+#endif /* UNIV_DEBUG */
+
 	page_check_dir(page);
 
 #ifdef PAGE_CUR_ADAPT
@@ -229,7 +258,7 @@ page_cur_search_with_match(
 	    && (page_header_get_ptr(page, PAGE_LAST_INSERT))
 	    && (page_header_get_field(page, PAGE_DIRECTION) == PAGE_RIGHT)) {
 
-		if (page_cur_try_search_shortcut(page, tuple,
+		if (page_cur_try_search_shortcut(page, index, tuple,
 	    					iup_matched_fields,
 	    					iup_matched_bytes,
 	    					ilow_matched_fields,
@@ -238,16 +267,18 @@ page_cur_search_with_match(
 	    		return;
 	    	}
 	}
-/*#ifdef UNIV_SEARCH_DEBUG */
+# ifdef PAGE_CUR_DBG
 	if (mode == PAGE_CUR_DBG) {
 		mode = PAGE_CUR_LE;
 	}
-/*#endif */
+# endif
 #endif	
 
 	/* The following flag does not work for non-latin1 char sets because
 	cmp_full_field does not tell how many bytes matched */
+#ifdef PAGE_CUR_LE_OR_EXTENDS
 	ut_a(mode != PAGE_CUR_LE_OR_EXTENDS); 
+#endif /* PAGE_CUR_LE_OR_EXTENDS */
 
 	/* If mode PAGE_CUR_G is specified, we are trying to position the
 	cursor to answer a query of the form "tuple < X", where tuple is
@@ -279,37 +310,42 @@ page_cur_search_with_match(
 				low_matched_fields, low_matched_bytes,
 				up_matched_fields, up_matched_bytes);
 
-		cmp = cmp_dtuple_rec_with_match(tuple, mid_rec,
+		offsets = rec_get_offsets(mid_rec, index, offsets,
+					dtuple_get_n_fields_cmp(tuple), &heap);
+
+		cmp = cmp_dtuple_rec_with_match(tuple, mid_rec, offsets,
 						&cur_matched_fields,
 						&cur_matched_bytes);
-		if (cmp == 1) {
+		if (UNIV_LIKELY(cmp > 0)) {
+low_slot_match:
 			low = mid;
 			low_matched_fields = cur_matched_fields;
 			low_matched_bytes = cur_matched_bytes;
 
-		} else if (cmp == -1) {
-
+		} else if (UNIV_LIKELY(cmp /* == -1 */)) {
+#ifdef PAGE_CUR_LE_OR_EXTENDS
 			if (mode == PAGE_CUR_LE_OR_EXTENDS
 			    && page_cur_rec_field_extends(tuple, mid_rec,
-						     cur_matched_fields)) {
-				low = mid;
-				low_matched_fields = cur_matched_fields;
-				low_matched_bytes = cur_matched_bytes;
-			} else {
-				up = mid;
-				up_matched_fields = cur_matched_fields;
-				up_matched_bytes = cur_matched_bytes;
-			}
+						offsets, cur_matched_fields)) {
 
-		} else if (mode == PAGE_CUR_G || mode == PAGE_CUR_LE
-			   || mode == PAGE_CUR_LE_OR_EXTENDS) {
-			low = mid;
-			low_matched_fields = cur_matched_fields;
-			low_matched_bytes = cur_matched_bytes;
-		} else {
+				goto low_slot_match;
+			}
+#endif /* PAGE_CUR_LE_OR_EXTENDS */
+up_slot_match:
 			up = mid;
 			up_matched_fields = cur_matched_fields;
 			up_matched_bytes = cur_matched_bytes;
+
+		} else if (mode == PAGE_CUR_G || mode == PAGE_CUR_LE
+#ifdef PAGE_CUR_LE_OR_EXTENDS
+			   || mode == PAGE_CUR_LE_OR_EXTENDS
+#endif /* PAGE_CUR_LE_OR_EXTENDS */
+		) {
+
+			goto low_slot_match;
+		} else {
+
+			goto up_slot_match;
 		}
    	}
 
@@ -329,35 +365,41 @@ page_cur_search_with_match(
 				low_matched_fields, low_matched_bytes,
 				up_matched_fields, up_matched_bytes);
 
-		cmp = cmp_dtuple_rec_with_match(tuple, mid_rec,
+		offsets = rec_get_offsets(mid_rec, index, offsets,
+					dtuple_get_n_fields_cmp(tuple), &heap);
+
+		cmp = cmp_dtuple_rec_with_match(tuple, mid_rec, offsets,
 						&cur_matched_fields,
 						&cur_matched_bytes);
-		if (cmp == 1) {
+		if (UNIV_LIKELY(cmp > 0)) {
+low_rec_match:
 			low_rec = mid_rec;
 			low_matched_fields = cur_matched_fields;
 			low_matched_bytes = cur_matched_bytes;
 
-		} else if (cmp == -1) {
+		} else if (UNIV_LIKELY(cmp /* == -1 */)) {
+#ifdef PAGE_CUR_LE_OR_EXTENDS
 			if (mode == PAGE_CUR_LE_OR_EXTENDS
 			    && page_cur_rec_field_extends(tuple, mid_rec,
-						     cur_matched_fields)) {
-				low_rec = mid_rec;
-				low_matched_fields = cur_matched_fields;
-				low_matched_bytes = cur_matched_bytes;
-			} else {
-				up_rec = mid_rec;
-				up_matched_fields = cur_matched_fields;
-				up_matched_bytes = cur_matched_bytes;
+						offsets, cur_matched_fields)) {
+
+				goto low_rec_match;
 			}
-		} else if (mode == PAGE_CUR_G || mode == PAGE_CUR_LE
-			   || mode == PAGE_CUR_LE_OR_EXTENDS) {
-			low_rec = mid_rec;
-			low_matched_fields = cur_matched_fields;
-			low_matched_bytes = cur_matched_bytes;
-		} else {
+#endif /* PAGE_CUR_LE_OR_EXTENDS */
+up_rec_match:
 			up_rec = mid_rec;
 			up_matched_fields = cur_matched_fields;
 			up_matched_bytes = cur_matched_bytes;
+		} else if (mode == PAGE_CUR_G || mode == PAGE_CUR_LE
+#ifdef PAGE_CUR_LE_OR_EXTENDS
+			   || mode == PAGE_CUR_LE_OR_EXTENDS
+#endif /* PAGE_CUR_LE_OR_EXTENDS */
+		) {
+
+			goto low_rec_match;
+		} else {
+
+			goto up_rec_match;
 		}
    	}
 
@@ -368,7 +410,9 @@ page_cur_search_with_match(
 	dbg_matched_fields = 0;
 	dbg_matched_bytes = 0;
 
-	dbg_cmp = page_cmp_dtuple_rec_with_match(tuple, low_rec,
+	offsets = rec_get_offsets(low_rec, index, offsets,
+						ULINT_UNDEFINED, &heap);
+	dbg_cmp = page_cmp_dtuple_rec_with_match(tuple, low_rec, offsets,
 						&dbg_matched_fields,
 						&dbg_matched_bytes);
 	if (mode == PAGE_CUR_G) {
@@ -390,7 +434,9 @@ page_cur_search_with_match(
 	dbg_matched_fields = 0;
 	dbg_matched_bytes = 0;
 
-	dbg_cmp = page_cmp_dtuple_rec_with_match(tuple, up_rec,
+	offsets = rec_get_offsets(up_rec, index, offsets,
+						ULINT_UNDEFINED, &heap);
+	dbg_cmp = page_cmp_dtuple_rec_with_match(tuple, up_rec, offsets,
 						&dbg_matched_fields,
 						&dbg_matched_bytes);
 	if (mode == PAGE_CUR_G) {
@@ -419,6 +465,9 @@ page_cur_search_with_match(
 	*iup_matched_bytes   = up_matched_bytes;
 	*ilow_matched_fields = low_matched_fields;
 	*ilow_matched_bytes  = low_matched_bytes;
+	if (UNIV_LIKELY_NULL(heap)) {
+		mem_heap_free(heap);
+	}
 }
 
 /***************************************************************
@@ -463,10 +512,12 @@ static
 void
 page_cur_insert_rec_write_log(
 /*==========================*/
-	rec_t*	insert_rec,	/* in: inserted physical record */
-	ulint	rec_size,	/* in: insert_rec size */
-	rec_t*	cursor_rec,	/* in: record the cursor is pointing to */
-	mtr_t*	mtr)		/* in: mini-transaction handle */
+	rec_t*		insert_rec,	/* in: inserted physical record */
+	ulint		rec_size,	/* in: insert_rec size */
+	rec_t*		cursor_rec,	/* in: record the
+					cursor is pointing to */
+	dict_index_t*	index,		/* in: record descriptor */
+	mtr_t*		mtr)		/* in: mini-transaction handle */
 {
 	ulint	cur_rec_size;
 	ulint	extra_size;
@@ -476,23 +527,41 @@ page_cur_insert_rec_write_log(
 	byte*	cur_ptr;
 	ulint	extra_info_yes;
 	byte*	log_ptr;
+	byte*	log_end;
 	ulint	i;
+	ulint	comp;
 
 	ut_a(rec_size < UNIV_PAGE_SIZE);
-	ut_ad(rec_size == rec_get_size(insert_rec));
+	ut_ad(buf_frame_align(insert_rec) == buf_frame_align(cursor_rec));
+	ut_ad(!page_rec_is_comp(insert_rec) == !index->table->comp);
+	comp = page_rec_is_comp(insert_rec);
 
-	log_ptr = mlog_open(mtr, 30 + MLOG_BUF_MARGIN);
+	{
+		mem_heap_t*	heap		= NULL;
+		ulint		cur_offs_[REC_OFFS_NORMAL_SIZE];
+		ulint		ins_offs_[REC_OFFS_NORMAL_SIZE];
 
-	if (log_ptr == NULL) {
+		ulint*		cur_offs;
+		ulint*		ins_offs;
 
-		return;
+		*cur_offs_ = (sizeof cur_offs_) / sizeof *cur_offs_;
+		*ins_offs_ = (sizeof ins_offs_) / sizeof *ins_offs_;
+
+		cur_offs = rec_get_offsets(cursor_rec, index, cur_offs_,
+						ULINT_UNDEFINED, &heap);
+		ins_offs = rec_get_offsets(insert_rec, index, ins_offs_,
+						ULINT_UNDEFINED, &heap);
+
+		extra_size = rec_offs_extra_size(ins_offs);
+		cur_extra_size = rec_offs_extra_size(cur_offs);
+		ut_ad(rec_size == rec_offs_size(ins_offs));
+		cur_rec_size = rec_offs_size(cur_offs);
+
+		if (UNIV_LIKELY_NULL(heap)) {
+			mem_heap_free(heap);
+		}
 	}
 
-	extra_size = rec_get_extra_size(insert_rec);
-
-	cur_extra_size = rec_get_extra_size(cursor_rec);
-	cur_rec_size = rec_get_size(cursor_rec);
-
 	ins_ptr = insert_rec - extra_size;
 
 	i = 0;
@@ -514,7 +583,9 @@ page_cur_insert_rec_write_log(
 				ins_ptr++;
 				cur_ptr++;
 			} else if ((i < extra_size)
-				   && (i >= extra_size - REC_N_EXTRA_BYTES)) {
+				   && (i >= extra_size - (comp
+					? REC_N_NEW_EXTRA_BYTES
+					: REC_N_OLD_EXTRA_BYTES))) {
 				i = extra_size;
 				ins_ptr = insert_rec;
 				cur_ptr = cursor_rec;
@@ -525,16 +596,35 @@ page_cur_insert_rec_write_log(
 	}
 
 	if (mtr_get_log_mode(mtr) != MTR_LOG_SHORT_INSERTS) {
-	
-		log_ptr = mlog_write_initial_log_record_fast(insert_rec,
-					MLOG_REC_INSERT, log_ptr, mtr);
+
+		log_ptr = mlog_open_and_write_index(mtr, insert_rec, index,
+				comp
+				? MLOG_COMP_REC_INSERT : MLOG_REC_INSERT,
+				2 + 5 + 1 + 5 + 5 + MLOG_BUF_MARGIN);
+
+		if (!log_ptr) {
+			/* Logging in mtr is switched off during crash
+			recovery: in that case mlog_open returns NULL */
+			return;
+		}
+
+		log_end = &log_ptr[2 + 5 + 1 + 5 + 5 + MLOG_BUF_MARGIN];
 		/* Write the cursor rec offset as a 2-byte ulint */
 		mach_write_to_2(log_ptr, cursor_rec
 					- buf_frame_align(cursor_rec));
 		log_ptr += 2;
+	} else {
+		log_ptr = mlog_open(mtr, 5 + 1 + 5 + 5 + MLOG_BUF_MARGIN);
+		if (!log_ptr) {
+			/* Logging in mtr is switched off during crash
+			recovery: in that case mlog_open returns NULL */
+			return;
+		}
+		log_end = &log_ptr[5 + 1 + 5 + 5 + MLOG_BUF_MARGIN];
 	}
 
-	if ((rec_get_info_bits(insert_rec) != rec_get_info_bits(cursor_rec))
+	if ((rec_get_info_and_status_bits(insert_rec, comp) !=
+	     rec_get_info_and_status_bits(cursor_rec, comp))
 	    || (extra_size != cur_extra_size)
 	    || (rec_size != cur_rec_size)) {
 
@@ -549,7 +639,8 @@ page_cur_insert_rec_write_log(
 							+ extra_info_yes);
 	if (extra_info_yes) {
 		/* Write the info bits */
-		mach_write_to_1(log_ptr, rec_get_info_bits(insert_rec));
+		mach_write_to_1(log_ptr,
+			rec_get_info_and_status_bits(insert_rec, comp));
 		log_ptr++;
 
 		/* Write the record origin offset */
@@ -565,17 +656,15 @@ page_cur_insert_rec_write_log(
 	/* Write to the log the inserted index record end segment which
 	differs from the cursor record */
 
-	if (rec_size - i < MLOG_BUF_MARGIN) {
-		ut_memcpy(log_ptr, ins_ptr, rec_size - i);
-		log_ptr += rec_size - i;
-	}
+	rec_size -= i;
 
-	mlog_close(mtr, log_ptr);
-
-	ut_a(rec_size - i < UNIV_PAGE_SIZE);
-
-	if (rec_size - i >= MLOG_BUF_MARGIN) {
-		mlog_catenate_string(mtr, ins_ptr, rec_size - i);
+	if (log_ptr + rec_size <= log_end) {
+		memcpy(log_ptr, ins_ptr, rec_size);
+		mlog_close(mtr, log_ptr + rec_size);
+	} else {
+		mlog_close(mtr, log_ptr);
+		ut_a(rec_size < UNIV_PAGE_SIZE);
+		mlog_catenate_string(mtr, ins_ptr, rec_size);
 	}
 }	
 
@@ -585,12 +674,13 @@ Parses a log record of a record insert on a page. */
 byte*
 page_cur_parse_insert_rec(
 /*======================*/
-			/* out: end of log record or NULL */
-	ibool	is_short,/* in: TRUE if short inserts */
-	byte*	ptr,	/* in: buffer */
-	byte*	end_ptr,/* in: buffer end */
-	page_t*	page,	/* in: page or NULL */
-	mtr_t*	mtr)	/* in: mtr or NULL */
+				/* out: end of log record or NULL */
+	ibool		is_short,/* in: TRUE if short inserts */
+	byte*		ptr,	/* in: buffer */
+	byte*		end_ptr,/* in: buffer end */
+	dict_index_t*	index,	/* in: record descriptor */
+	page_t*		page,	/* in: page or NULL */
+	mtr_t*		mtr)	/* in: mtr or NULL */
 {
 	ulint	extra_info_yes;
 	ulint	offset = 0; /* remove warning */
@@ -601,8 +691,12 @@ page_cur_parse_insert_rec(
 	byte	buf1[1024];
 	byte*	buf;
 	byte*   ptr2 = ptr;
-	ulint	info_bits = 0; /* remove warning */
+	ulint	info_and_status_bits = 0; /* remove warning */
 	page_cur_t cursor;
+	mem_heap_t*	heap		= NULL;
+	ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+	ulint*		offsets		= offsets_;
+	*offsets_ = (sizeof offsets_) / sizeof *offsets_;
 
 	if (!is_short) {
 		/* Read the cursor rec offset as a 2-byte ulint */
@@ -648,7 +742,7 @@ page_cur_parse_insert_rec(
 			return(NULL);
 		}
 		
-		info_bits = mach_read_from_1(ptr);
+		info_and_status_bits = mach_read_from_1(ptr);
 		ptr++;
 
 		ptr = mach_parse_compressed(ptr, end_ptr, &origin_offset);
@@ -680,6 +774,8 @@ page_cur_parse_insert_rec(
 		return(ptr + end_seg_len);
 	}
 
+	ut_ad(!!page_is_comp(page) == index->table->comp);
+
 	/* Read from the log the inserted index record end segment which
 	differs from the cursor record */
 
@@ -689,11 +785,15 @@ page_cur_parse_insert_rec(
 		cursor_rec = page + offset;
 	}
 
+	offsets = rec_get_offsets(cursor_rec, index, offsets,
+						ULINT_UNDEFINED, &heap);
+
 	if (extra_info_yes == 0) {
-		info_bits = rec_get_info_bits(cursor_rec);
-		origin_offset = rec_get_extra_size(cursor_rec);
-		mismatch_index = rec_get_size(cursor_rec) - end_seg_len;
-	} 
+		info_and_status_bits = rec_get_info_and_status_bits(
+					cursor_rec, page_is_comp(page));
+		origin_offset = rec_offs_extra_size(offsets);
+		mismatch_index = rec_offs_size(offsets) - end_seg_len;
+	}
 	
 	if (mismatch_index + end_seg_len < sizeof buf1) {
 		buf = buf1;
@@ -705,12 +805,15 @@ page_cur_parse_insert_rec(
 	
         if (mismatch_index >= UNIV_PAGE_SIZE) {
 		fprintf(stderr,
-			"Is short %lu, info_bits %lu, offset %lu, "
+			"Is short %lu, info_and_status_bits %lu, offset %lu, "
 			"o_offset %lu\n"
                     "mismatch index %lu, end_seg_len %lu\n"
                     "parsed len %lu\n",
-                    is_short, info_bits, offset, origin_offset,
-                    mismatch_index, end_seg_len, (ulint)(ptr - ptr2));
+		    (ulong) is_short, (ulong) info_and_status_bits,
+		    (ulong) offset,
+		    (ulong) origin_offset,
+		    (ulong) mismatch_index, (ulong) end_seg_len,
+		    (ulong) (ptr - ptr2));
 
 		fputs("Dump of 300 bytes of log:\n", stderr);
 		ut_print_buf(stderr, ptr2, 300);
@@ -720,20 +823,27 @@ page_cur_parse_insert_rec(
 	       ut_error;
 	}
 
-	ut_memcpy(buf, rec_get_start(cursor_rec), mismatch_index);
+	ut_memcpy(buf, rec_get_start(cursor_rec, offsets), mismatch_index);
 	ut_memcpy(buf + mismatch_index, ptr, end_seg_len);
 
-	rec_set_info_bits(buf + origin_offset, info_bits);
+	rec_set_info_and_status_bits(buf + origin_offset, page_is_comp(page),
+							info_and_status_bits);
 
 	page_cur_position(cursor_rec, &cursor);
 
-	page_cur_rec_insert(&cursor, buf + origin_offset, mtr);
+	offsets = rec_get_offsets(buf + origin_offset, index, offsets,
+						ULINT_UNDEFINED, &heap);
+	page_cur_rec_insert(&cursor, buf + origin_offset, index, offsets, mtr);
 
 	if (buf != buf1) {
 
 		mem_free(buf);
 	}
 
+	if (UNIV_LIKELY_NULL(heap)) {
+		mem_heap_free(heap);
+	}
+
 	return(ptr + end_seg_len);
 }	
 
@@ -749,68 +859,85 @@ page_cur_insert_rec_low(
 				/* out: pointer to record if succeed, NULL
 				otherwise */
 	page_cur_t*	cursor,	/* in: a page cursor */
-	dtuple_t*      	tuple,  /* in: pointer to a data tuple or NULL */
-	ulint		data_size,/* in: data size of tuple */
-	rec_t*      	rec,  	/* in: pointer to a physical record or NULL */
+	dtuple_t*	tuple,	/* in: pointer to a data tuple or NULL */
+	dict_index_t*	index,	/* in: record descriptor */
+	rec_t*		rec,	/* in: pointer to a physical record or NULL */
+	ulint*		offsets,/* in: rec_get_offsets(rec, index) or NULL */
 	mtr_t*		mtr)	/* in: mini-transaction handle */
 {
-	byte*	insert_buf	= NULL;
-	ulint	rec_size;
-	byte*	page;		/* the relevant page */
-	rec_t*	last_insert;	/* cursor position at previous insert */
-	rec_t*	insert_rec;	/* inserted record */
-	ulint	heap_no;	/* heap number of the inserted record */
-	rec_t*	current_rec;	/* current record after which the
-				new record is inserted */
-	rec_t*	next_rec;	/* next record after current before
-				the insertion */
-	ulint	owner_slot;	/* the slot which owns the inserted record */
-	rec_t*	owner_rec;	
-	ulint	n_owned;
-	
+	byte*		insert_buf	= NULL;
+	ulint		rec_size;
+	byte*		page;		/* the relevant page */
+	rec_t*		last_insert;	/* cursor position at previous insert */
+	rec_t*		insert_rec;	/* inserted record */
+	ulint		heap_no;	/* heap number of the inserted record */
+	rec_t*		current_rec;	/* current record after which the
+					new record is inserted */
+	rec_t*		next_rec;	/* next record after current before
+					the insertion */
+	ulint		owner_slot;	/* the slot which owns the
+					inserted record */
+	rec_t*		owner_rec;
+	ulint		n_owned;
+	mem_heap_t*	heap		= NULL;
+	ulint		comp;
+
 	ut_ad(cursor && mtr);
 	ut_ad(tuple || rec);
 	ut_ad(!(tuple && rec));
 	ut_ad(rec || dtuple_check_typed(tuple));
-	ut_ad(rec || (dtuple_get_data_size(tuple) == data_size));
 
 	page = page_cur_get_page(cursor);
+	comp = page_is_comp(page);
+	ut_ad(index->table->comp == !!comp);
 
 	ut_ad(cursor->rec != page_get_supremum_rec(page));	
 
 	/* 1. Get the size of the physical record in the page */
 	if (tuple != NULL) {
-		rec_size = data_size + rec_get_converted_extra_size(
-						data_size,
-						dtuple_get_n_fields(tuple));
+		rec_size = rec_get_converted_size(index, tuple);
 	} else {
-		rec_size = rec_get_size(rec);
+		if (!offsets) {
+			offsets = rec_get_offsets(rec, index, offsets,
+						ULINT_UNDEFINED, &heap);
+		}
+		ut_ad(rec_offs_validate(rec, index, offsets));
+		rec_size = rec_offs_size(offsets);
 	}
 
 	/* 2. Try to find suitable space from page memory management */
-	insert_buf = page_mem_alloc(page, rec_size, &heap_no);
+	insert_buf = page_mem_alloc(page, rec_size, index, &heap_no);
 
 	if (insert_buf == NULL) {
-
+		if (UNIV_LIKELY_NULL(heap)) {
+			mem_heap_free(heap);
+		}
 		return(NULL);
 	}
 
 	/* 3. Create the record */
 	if (tuple != NULL) {
-		insert_rec = rec_convert_dtuple_to_rec_low(insert_buf, tuple,
-								data_size);
+		insert_rec = rec_convert_dtuple_to_rec(insert_buf,
+							index, tuple);
+		offsets = rec_get_offsets(insert_rec, index, offsets,
+						ULINT_UNDEFINED, &heap);
 	} else {
-		insert_rec = rec_copy(insert_buf, rec);
+		insert_rec = rec_copy(insert_buf, rec, offsets);
+		ut_ad(rec_offs_validate(rec, index, offsets));
+		rec_offs_make_valid(insert_rec, index, offsets);
 	}
 
 	ut_ad(insert_rec);
-	ut_ad(rec_size == rec_get_size(insert_rec));
+	ut_ad(rec_size == rec_offs_size(offsets));
 	
 	/* 4. Insert the record in the linked list of records */
-	
 	current_rec = cursor->rec;
 
+	ut_ad(!comp || rec_get_status(current_rec) <= REC_STATUS_INFIMUM);
+	ut_ad(!comp || rec_get_status(insert_rec) < REC_STATUS_INFIMUM);
+
 	next_rec = page_rec_get_next(current_rec);
+	ut_ad(!comp || rec_get_status(next_rec) != REC_STATUS_INFIMUM);
 	page_rec_set_next(insert_rec, next_rec);
 	page_rec_set_next(current_rec, insert_rec);
 
@@ -819,12 +946,15 @@ page_cur_insert_rec_low(
 	/* 5. Set the n_owned field in the inserted record to zero,
 	and set the heap_no field */	
 	
-	rec_set_n_owned(insert_rec, 0);
-	rec_set_heap_no(insert_rec, heap_no);
+	rec_set_n_owned(insert_rec, comp, 0);
+	rec_set_heap_no(insert_rec, comp, heap_no);
 
 	/* 6. Update the last insertion info in page header */	
 
 	last_insert = page_header_get_ptr(page, PAGE_LAST_INSERT);
+	ut_ad(!last_insert || !comp
+		|| rec_get_node_ptr_flag(last_insert)
+		== rec_get_node_ptr_flag(insert_rec));
 
 	if (last_insert == NULL) {
 	    	page_header_set_field(page, PAGE_DIRECTION, PAGE_NO_DIRECTION);
@@ -853,8 +983,8 @@ page_cur_insert_rec_low(
 	/* 7. It remains to update the owner record. */		
 	
 	owner_rec = page_rec_find_owner_rec(insert_rec);
-	n_owned = rec_get_n_owned(owner_rec);
-	rec_set_n_owned(owner_rec, n_owned + 1);
+	n_owned = rec_get_n_owned(owner_rec, comp);
+	rec_set_n_owned(owner_rec, comp, n_owned + 1);
 
 	/* 8. Now we have incremented the n_owned field of the owner
 	record. If the number exceeds PAGE_DIR_SLOT_MAX_N_OWNED,
@@ -866,8 +996,12 @@ page_cur_insert_rec_low(
 	}
 
 	/* 9. Write log record of the insert */
- 	page_cur_insert_rec_write_log(insert_rec, rec_size, current_rec, mtr);
+	page_cur_insert_rec_write_log(insert_rec, rec_size, current_rec,
+				index, mtr);
 	
+	if (UNIV_LIKELY_NULL(heap)) {
+		mem_heap_free(heap);
+	}
 	return(insert_rec);
 }
 
@@ -877,17 +1011,21 @@ UNIV_INLINE
 byte*
 page_copy_rec_list_to_created_page_write_log(
 /*=========================================*/
-			/* out: 4-byte field where to write the log data
-			length */
-	page_t*	page,	/* in: index page */
-	mtr_t*	mtr)	/* in: mtr */
+				/* out: 4-byte field where to
+				write the log data length */
+	page_t*		page,	/* in: index page */
+	dict_index_t*	index,	/* in: record descriptor */
+	mtr_t*		mtr)	/* in: mtr */
 {
 	byte*	log_ptr;
-	
-	mlog_write_initial_log_record(page, MLOG_LIST_END_COPY_CREATED, mtr);
 
-	log_ptr = mlog_open(mtr, 4);
+	ut_ad(!!page_is_comp(page) == index->table->comp);
 
+	log_ptr = mlog_open_and_write_index(mtr, page, index,
+			page_is_comp(page)
+			? MLOG_COMP_LIST_END_COPY_CREATED
+			: MLOG_LIST_END_COPY_CREATED, 4);
+	ut_a(log_ptr);
 	mlog_close(mtr, log_ptr + 4);
 
 	return(log_ptr);
@@ -899,11 +1037,12 @@ Parses a log record of copying a record list end to a new created page. */
 byte*
 page_parse_copy_rec_list_to_created_page(
 /*=====================================*/
-			/* out: end of log record or NULL */
-	byte*	ptr,	/* in: buffer */
-	byte*	end_ptr,/* in: buffer end */
-	page_t*	page,	/* in: page or NULL */
-	mtr_t*	mtr)	/* in: mtr or NULL */
+				/* out: end of log record or NULL */
+	byte*		ptr,	/* in: buffer */
+	byte*		end_ptr,/* in: buffer end */
+	dict_index_t*	index,	/* in: record descriptor */
+	page_t*		page,	/* in: page or NULL */
+	mtr_t*		mtr)	/* in: mtr or NULL */
 {
 	byte*	rec_end;
 	ulint	log_data_len;
@@ -929,7 +1068,8 @@ page_parse_copy_rec_list_to_created_page(
 	}
 
 	while (ptr < rec_end) {
-		ptr = page_cur_parse_insert_rec(TRUE, ptr, end_ptr, page, mtr);
+		ptr = page_cur_parse_insert_rec(TRUE, ptr, end_ptr,
+							index, page, mtr);
 	}
 
 	ut_a(ptr == rec_end);
@@ -948,10 +1088,11 @@ including that record. Infimum and supremum records are not copied. */
 void
 page_copy_rec_list_end_to_created_page(
 /*===================================*/
-	page_t*	new_page,	/* in: index page to copy to */
-	page_t*	page,		/* in: index page */
-	rec_t*	rec,		/* in: first record to copy */
-	mtr_t*	mtr)		/* in: mtr */
+	page_t*		new_page,	/* in: index page to copy to */
+	page_t*		page,		/* in: index page */
+	rec_t*		rec,		/* in: first record to copy */
+	dict_index_t*	index,		/* in: record descriptor */
+	mtr_t*		mtr)		/* in: mtr */
 {
 	page_dir_slot_t* slot = 0; /* remove warning */
 	byte*	heap_top;
@@ -964,9 +1105,15 @@ page_copy_rec_list_end_to_created_page(
 	ulint	log_mode;
 	byte*	log_ptr;
 	ulint	log_data_len;
+	ulint		comp		= page_is_comp(page);
+	mem_heap_t*	heap		= NULL;
+	ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+	ulint*		offsets		= offsets_;
+	*offsets_ = (sizeof offsets_) / sizeof *offsets_;
 	
-	ut_ad(page_header_get_field(new_page, PAGE_N_HEAP) == 2);
+	ut_ad(page_dir_get_n_heap(new_page) == 2);
 	ut_ad(page != new_page);
+	ut_ad(comp == page_is_comp(new_page));
 	
 	if (rec == page_get_infimum_rec(page)) {
 
@@ -981,12 +1128,13 @@ page_copy_rec_list_end_to_created_page(
 #ifdef UNIV_DEBUG
 	/* To pass the debug tests we have to set these dummy values
 	in the debug version */
-	page_header_set_field(new_page, PAGE_N_DIR_SLOTS, UNIV_PAGE_SIZE / 2);
+	page_dir_set_n_slots(new_page, UNIV_PAGE_SIZE / 2);
 	page_header_set_ptr(new_page, PAGE_HEAP_TOP,
 					new_page + UNIV_PAGE_SIZE - 1);
 #endif
 
-	log_ptr = page_copy_rec_list_to_created_page_write_log(new_page, mtr);
+	log_ptr = page_copy_rec_list_to_created_page_write_log(new_page,
+								index, mtr);
 
 	log_data_len = dyn_array_get_data_size(&(mtr->log));
 
@@ -995,22 +1143,27 @@ page_copy_rec_list_end_to_created_page(
 	log_mode = mtr_set_log_mode(mtr, MTR_LOG_SHORT_INSERTS);
 	
 	prev_rec = page_get_infimum_rec(new_page);
-	heap_top = new_page + PAGE_SUPREMUM_END;
+	if (comp) {
+		heap_top = new_page + PAGE_NEW_SUPREMUM_END;
+	} else {
+		heap_top = new_page + PAGE_OLD_SUPREMUM_END;
+	}
 	count = 0;
 	slot_index = 0;
 	n_recs = 0;
 
 	/* should be do ... until, comment by Jani */
 	while (rec != page_get_supremum_rec(page)) {
-		
-		insert_rec = rec_copy(heap_top, rec);
+		offsets = rec_get_offsets(rec, index, offsets,
+					ULINT_UNDEFINED, &heap);
+		insert_rec = rec_copy(heap_top, rec, offsets);
 
-		rec_set_next_offs(prev_rec, insert_rec - new_page);
+		rec_set_next_offs(prev_rec, comp, insert_rec - new_page);
 
-		rec_set_n_owned(insert_rec, 0);
-		rec_set_heap_no(insert_rec, 2 + n_recs);
+		rec_set_n_owned(insert_rec, comp, 0);
+		rec_set_heap_no(insert_rec, comp, 2 + n_recs);
 
-		rec_size = rec_get_size(insert_rec);
+		rec_size = rec_offs_size(offsets);
 
 		heap_top = heap_top + rec_size;
 		
@@ -1032,7 +1185,7 @@ page_copy_rec_list_end_to_created_page(
 		}
 		
  		page_cur_insert_rec_write_log(insert_rec, rec_size, prev_rec,
-									mtr);
+								index, mtr);
 		prev_rec = insert_rec;
 		rec = page_rec_get_next(rec);
 	}
@@ -1054,22 +1207,27 @@ page_copy_rec_list_end_to_created_page(
 		slot_index--;
 	}
 
+	if (UNIV_LIKELY_NULL(heap)) {
+		mem_heap_free(heap);
+	}
+
 	log_data_len = dyn_array_get_data_size(&(mtr->log)) - log_data_len;
 
 	ut_a(log_data_len < 100 * UNIV_PAGE_SIZE);
 
 	mach_write_to_4(log_ptr, log_data_len);
 	
-	rec_set_next_offs(insert_rec, PAGE_SUPREMUM);
+	rec_set_next_offs(insert_rec, comp,
+				comp ? PAGE_NEW_SUPREMUM : PAGE_OLD_SUPREMUM);
 
 	slot = page_dir_get_nth_slot(new_page, 1 + slot_index);
 
 	page_dir_slot_set_rec(slot, page_get_supremum_rec(new_page));
 	page_dir_slot_set_n_owned(slot, count + 1);
 
-	page_header_set_field(new_page, PAGE_N_DIR_SLOTS, 2 + slot_index);
+	page_dir_set_n_slots(new_page, 2 + slot_index);
 	page_header_set_ptr(new_page, PAGE_HEAP_TOP, heap_top);
-	page_header_set_field(new_page, PAGE_N_HEAP, 2 + n_recs);
+	page_dir_set_n_heap(new_page, 2 + n_recs);
 	page_header_set_field(new_page, PAGE_N_RECS, n_recs);
 	
 	page_header_set_ptr(new_page, PAGE_LAST_INSERT, NULL);
@@ -1087,14 +1245,29 @@ UNIV_INLINE
 void
 page_cur_delete_rec_write_log(
 /*==========================*/
-	rec_t*	cursor_rec,	/* in: record to be deleted */
-	mtr_t*	mtr)		/* in: mini-transaction handle */
+	rec_t*		rec,	/* in: record to be deleted */
+	dict_index_t*	index,	/* in: record descriptor */
+	mtr_t*		mtr)	/* in: mini-transaction handle */
 {
-	mlog_write_initial_log_record(cursor_rec, MLOG_REC_DELETE, mtr);
+	byte*	log_ptr;
+
+	ut_ad(!!page_rec_is_comp(rec) == index->table->comp);
+
+	log_ptr = mlog_open_and_write_index(mtr, rec, index,
+			page_rec_is_comp(rec)
+			? MLOG_COMP_REC_DELETE
+			: MLOG_REC_DELETE, 2);
+
+	if (!log_ptr) {
+		/* Logging in mtr is switched off during crash recovery:
+		in that case mlog_open returns NULL */
+		return;
+	}
 
 	/* Write the cursor rec offset as a 2-byte ulint */
-	mlog_catenate_ulint(mtr, cursor_rec - buf_frame_align(cursor_rec),
-								MLOG_2BYTES);
+	mach_write_to_2(log_ptr, ut_align_offset(rec, UNIV_PAGE_SIZE));
+
+	mlog_close(mtr, log_ptr + 2);
 }	
 
 /***************************************************************
@@ -1103,11 +1276,12 @@ Parses log record of a record delete on a page. */
 byte*
 page_cur_parse_delete_rec(
 /*======================*/
-			/* out: pointer to record end or NULL */
-	byte*	ptr,	/* in: buffer */
-	byte*	end_ptr,/* in: buffer end */
-	page_t*	page,	/* in: page or NULL */
-	mtr_t*	mtr)	/* in: mtr or NULL */
+				/* out: pointer to record end or NULL */
+	byte*		ptr,	/* in: buffer */
+	byte*		end_ptr,/* in: buffer end */
+	dict_index_t*	index,	/* in: record descriptor */
+	page_t*		page,	/* in: page or NULL */
+	mtr_t*		mtr)	/* in: mtr or NULL */
 {
 	ulint		offset;
 	page_cur_t	cursor;
@@ -1124,9 +1298,19 @@ page_cur_parse_delete_rec(
 	ut_a(offset <= UNIV_PAGE_SIZE);
 
 	if (page) {
-		page_cur_position(page + offset, &cursor);
+		mem_heap_t*	heap		= NULL;
+		ulint		offsets_[REC_OFFS_NORMAL_SIZE];
+		rec_t*		rec		= page + offset;
+		*offsets_ = (sizeof offsets_) / sizeof *offsets_;
 
-		page_cur_delete_rec(&cursor, mtr);
+		page_cur_position(rec, &cursor);
+
+		page_cur_delete_rec(&cursor, index,
+				rec_get_offsets(rec, index, offsets_,
+				ULINT_UNDEFINED, &heap), mtr);
+		if (UNIV_LIKELY_NULL(heap)) {
+			mem_heap_free(heap);
+		}
 	}
 
 	return(ptr);
@@ -1140,6 +1324,8 @@ void
 page_cur_delete_rec(
 /*================*/
 	page_cur_t*  	cursor,	/* in: a page cursor */
+	dict_index_t*	index,	/* in: record descriptor */
+	const ulint*	offsets,/* in: rec_get_offsets(cursor->rec, index) */
 	mtr_t*		mtr)	/* in: mini-transaction handle */
 {
 	page_dir_slot_t* cur_dir_slot;
@@ -1156,6 +1342,8 @@ page_cur_delete_rec(
 	
 	page = page_cur_get_page(cursor);
 	current_rec = cursor->rec;
+	ut_ad(rec_offs_validate(current_rec, index, offsets));
+	ut_ad(!!page_is_comp(page) == index->table->comp);
 
 	/* The record must not be the supremum or infimum record. */
 	ut_ad(current_rec != page_get_supremum_rec(page));	
@@ -1167,7 +1355,7 @@ page_cur_delete_rec(
 	cur_n_owned = page_dir_slot_get_n_owned(cur_dir_slot);
 
 	/* 0. Write the log record */
-	page_cur_delete_rec_write_log(current_rec, mtr);
+	page_cur_delete_rec_write_log(current_rec, index, mtr);
 
 	/* 1. Reset the last insert info in the page header and increment
 	the modify clock for the frame */
@@ -1221,7 +1409,7 @@ page_cur_delete_rec(
 	page_dir_slot_set_n_owned(cur_dir_slot, cur_n_owned - 1);
 
 	/* 6. Free the memory occupied by the record */
-	page_mem_free(page, current_rec);
+	page_mem_free(page, current_rec, offsets);
 
 	/* 7. Now we have decremented the number of owned records of the slot.
 	If the number drops below PAGE_DIR_SLOT_MIN_N_OWNED, we balance the
