@@ -44,6 +44,7 @@ Created 10/8/1995 Heikki Tuuri
 #include "buf0flu.h"
 #include "btr0sea.h"
 #include "dict0load.h"
+#include "dict0boot.h"
 #include "srv0start.h"
 #include "row0mysql.h"
 
@@ -65,7 +66,7 @@ ulint	srv_dml_needed_delay = 0;
 ibool	srv_lock_timeout_and_monitor_active = FALSE;
 ibool	srv_error_monitor_active = FALSE;
 
-char*	srv_main_thread_op_info = (char*) "";
+const char*	srv_main_thread_op_info = "";
 
 /* Server parameters which are read from the initfile */
 
@@ -73,8 +74,18 @@ char*	srv_main_thread_op_info = (char*) "";
 names, where the file name itself may also contain a path */
 
 char*	srv_data_home 	= NULL;
+#ifdef UNIV_LOG_ARCHIVE
 char*	srv_arch_dir 	= NULL;
+#endif /* UNIV_LOG_ARCHIVE */
 
+ibool	srv_file_per_table = FALSE;	/* store to its own file each table
+					created by an user; data dictionary
+					tables are in the system tablespace
+					0 */
+ibool   srv_locks_unsafe_for_binlog = FALSE; /* Place locks to records only 
+                                                i.e. do not use next-key locking
+                                                except on duplicate key checking and
+                                                foreign key checking */
 ulint	srv_n_data_files = 0;
 char**	srv_data_file_names = NULL;
 ulint*	srv_data_file_sizes = NULL;	/* size in database pages */ 
@@ -101,9 +112,8 @@ char**	srv_log_group_home_dirs = NULL;
 ulint	srv_n_log_groups	= ULINT_MAX;
 ulint	srv_n_log_files		= ULINT_MAX;
 ulint	srv_log_file_size	= ULINT_MAX;	/* size in database pages */ 
-ibool	srv_log_archive_on	= TRUE;
 ulint	srv_log_buffer_size	= ULINT_MAX;	/* size in database pages */ 
-ulint	srv_flush_log_at_trx_commit = 1;
+ulong	srv_flush_log_at_trx_commit = 1;
 
 byte	srv_latin1_ordering[256]	/* The sort order table of the latin1
 					character set. The following table is
@@ -143,16 +153,24 @@ byte	srv_latin1_ordering[256]	/* The sort order table of the latin1
 , 0xD8, 0x55, 0x55, 0x55, 0x59, 0x59, 0xDE, 0xFF
 };
 		
-ulint	srv_pool_size		= ULINT_MAX;	/* size in database pages;
-						MySQL originally sets this
-						value in bytes */ 
+ulint	srv_pool_size		= ULINT_MAX;	/* size in pages; MySQL inits
+						this to size in kilobytes but
+						we normalize this to pages in
+						srv_boot() */
+ulint	srv_awe_window_size 	= 0;		/* size in pages; MySQL inits
+						this to bytes, but we
+						normalize it to pages in
+						srv_boot() */
 ulint	srv_mem_pool_size	= ULINT_MAX;	/* size in bytes */ 
 ulint	srv_lock_table_size	= ULINT_MAX;
 
 ulint	srv_n_file_io_threads	= ULINT_MAX;
 
+#ifdef UNIV_LOG_ARCHIVE
+ibool	srv_log_archive_on	= FALSE;
 ibool	srv_archive_recovery	= 0;
 dulint	srv_archive_recovery_limit_lsn;
+#endif /* UNIV_LOG_ARCHIVE */
 
 ulint	srv_lock_wait_timeout	= 1024 * 1024 * 1024;
 
@@ -160,6 +178,8 @@ char*   srv_file_flush_method_str = NULL;
 ulint   srv_unix_file_flush_method = SRV_UNIX_FDATASYNC;
 ulint   srv_win_file_flush_method = SRV_WIN_IO_UNBUFFERED;
 
+ulint	srv_max_n_open_files	  = 300;
+
 /* The InnoDB main thread tries to keep the ratio of modified pages
 in the buffer pool to all database pages in the buffer pool smaller than
 the following number. But it is not guaranteed that the value stays below
@@ -167,6 +187,61 @@ that during a time of heavy update/insert activity. */
 
 ulong	srv_max_buf_pool_modified_pct	= 90;
 
+/* variable counts amount of data read in total (in bytes) */
+ulint srv_data_read = 0;
+
+/* here we count the amount of data written in total (in bytes) */
+ulint srv_data_written = 0;
+
+/* the number of the log write requests done */
+ulint srv_log_write_requests = 0;
+
+/* the number of physical writes to the log performed */
+ulint srv_log_writes = 0;
+
+/* amount of data written to the log files in bytes */
+ulint srv_os_log_written = 0;
+
+/* amount of writes being done to the log files */
+ulint srv_os_log_pending_writes = 0;
+
+/* we increase this counter, when there we don't have enough space in the
+log buffer and have to flush it */
+ulint srv_log_waits = 0;
+
+/* this variable counts the amount of times, when the doublewrite buffer
+was flushed */
+ulint srv_dblwr_writes = 0;
+
+/* here we store the number of pages that have been flushed to the
+doublewrite buffer */
+ulint srv_dblwr_pages_written = 0;
+
+/* in this variable we store the number of write requests issued */
+ulint srv_buf_pool_write_requests = 0;
+
+/* here we store the number of times when we had to wait for a free page
+in the buffer pool. It happens when the buffer pool is full and we need
+to make a flush, in order to be able to read or create a page. */
+ulint srv_buf_pool_wait_free = 0;
+
+/* variable to count the number of pages that were written from buffer
+pool to the disk */
+ulint srv_buf_pool_flushed = 0;
+
+/* variable to count the number of buffer pool reads that led to the
+reading of a disk page */
+ulint srv_buf_pool_reads = 0;
+
+/* variable to count the number of sequential read-aheads */
+ulint srv_read_ahead_seq = 0;
+
+/* variable to count the number of random read-aheads */
+ulint srv_read_ahead_rnd = 0;
+
+/* structure to pass status variables to MySQL */
+export_struc export_vars;
+
 /* If the following is != 0 we do not allow inserts etc. This protects
 the user from forgetting the innodb_force_recovery keyword to my.cnf */
 
@@ -183,9 +258,11 @@ threads waiting for locks are not counted into the number because otherwise
 we could get a deadlock. MySQL creates a thread for each user session, and
 semaphore contention and convoy problems can occur withput this restriction.
 Value 10 should be good if there are less than 4 processors + 4 disks in the
-computer. Bigger computers need bigger values. */
+computer. Bigger computers need bigger values. Value 0 will disable the
+concurrency check. */
 
-ulint	srv_thread_concurrency	= 8;
+ulong	srv_thread_concurrency	= 0;
+ulong   srv_commit_concurrency  = 0;
 
 os_fast_mutex_t	srv_conc_mutex;		/* this mutex protects srv_conc data
 					structures */
@@ -217,28 +294,38 @@ struct srv_conc_slot_struct{
 
 UT_LIST_BASE_NODE_T(srv_conc_slot_t)	srv_conc_queue;	/* queue of threads
 							waiting to get in */
-srv_conc_slot_t*			srv_conc_slots;	/* array of wait
+srv_conc_slot_t* srv_conc_slots;			/* array of wait
 							slots */
 
 /* Number of times a thread is allowed to enter InnoDB within the same
 SQL query after it has once got the ticket at srv_conc_enter_innodb */
-#define SRV_FREE_TICKETS_TO_ENTER	500
-
+#define SRV_FREE_TICKETS_TO_ENTER srv_n_free_tickets_to_enter
+#define SRV_THREAD_SLEEP_DELAY srv_thread_sleep_delay
 /*-----------------------*/
-/* If the following is set TRUE then we do not run purge and insert buffer
-merge to completion before shutdown */
-
-ibool	srv_fast_shutdown	= FALSE;
+/* If the following is set to 1 then we do not run purge and insert buffer
+merge to completion before shutdown. If it is set to 2, do not even flush the
+buffer pool to data files at the shutdown: we effectively 'crash'
+InnoDB (but lose no committed transactions). */
+ulint	srv_fast_shutdown	= 0;
 
 /* Generate a innodb_status.<pid> file */
 ibool	srv_innodb_status	= FALSE;
 
 ibool	srv_use_doublewrite_buf	= TRUE;
+ibool	srv_use_checksums = TRUE;
 
 ibool   srv_set_thread_priorities = TRUE;
 int     srv_query_thread_priority = 0;
+
+/* TRUE if the Address Windowing Extensions of Windows are used; then we must
+disable adaptive hash indexes */
+ibool	srv_use_awe			= FALSE;
+ibool	srv_use_adaptive_hash_indexes 	= TRUE;
+
 /*-------------------------------------------*/
-ulint	srv_n_spin_wait_rounds	= 20;
+ulong	srv_n_spin_wait_rounds	= 20;
+ulong	srv_n_free_tickets_to_enter = 500;
+ulong	srv_thread_sleep_delay = 10000;
 ulint	srv_spin_wait_delay	= 5;
 ibool	srv_priority_boost	= TRUE;
 
@@ -257,6 +344,12 @@ static ulint	srv_n_rows_updated_old		= 0;
 static ulint	srv_n_rows_deleted_old		= 0;
 static ulint	srv_n_rows_read_old		= 0;
 
+ulint		srv_n_lock_wait_count		= 0;
+ulint		srv_n_lock_wait_current_count	= 0;
+ib_longlong	srv_n_lock_wait_time		= 0;
+ulint		srv_n_lock_max_wait_time	= 0;
+
+
 /*
   Set the following to 0 if you want InnoDB to write messages on
   stderr on startup/shutdown
@@ -305,6 +398,18 @@ mutex_t	srv_innodb_monitor_mutex;
 mutex_t	srv_monitor_file_mutex;
 /* Temporary file for innodb monitor output */
 FILE*	srv_monitor_file;
+/* Mutex for locking srv_dict_tmpfile.
+This mutex has a very high rank; threads reserving it should not
+be holding any InnoDB latches. */
+mutex_t	srv_dict_tmpfile_mutex;
+/* Temporary file for output from the data dictionary */
+FILE*	srv_dict_tmpfile;
+/* Mutex for locking srv_misc_tmpfile.
+This mutex has a very low rank; threads reserving it should not
+acquire any further latches or sleep before releasing this one. */
+mutex_t	srv_misc_tmpfile_mutex;
+/* Temporary file for miscellanous diagnostic output */
+FILE*	srv_misc_tmpfile;
 
 ulint	srv_main_thread_process_no	= 0;
 ulint	srv_main_thread_id		= 0;
@@ -643,8 +748,8 @@ srv_suspend_thread(void)
 	if (srv_print_thread_releases) {
 		fprintf(stderr,
 			"Suspending thread %lu to slot %lu meter %lu\n",
-			os_thread_get_curr_id(), slot_no,
-			srv_meter[SRV_RECOVERY]);
+			(ulong) os_thread_get_curr_id(), (ulong) slot_no,
+			(ulong) srv_meter[SRV_RECOVERY]);
 	}
 
 	slot = srv_table_get_nth_slot(slot_no);
@@ -706,7 +811,8 @@ srv_release_threads(
 			if (srv_print_thread_releases) {
 				fprintf(stderr,
 		"Releasing thread %lu type %lu from slot %lu meter %lu\n",
-				slot->id, type, i, srv_meter[SRV_RECOVERY]);
+				(ulong) slot->id, (ulong) type, (ulong) i,
+		                (ulong) srv_meter[SRV_RECOVERY]);
 			}
 
 			count++;
@@ -750,13 +856,14 @@ srv_get_thread_type(void)
 
 /*************************************************************************
 Initializes the server. */
-static
+
 void
 srv_init(void)
 /*==========*/
 {
 	srv_conc_slot_t* 	conc_slot;
 	srv_slot_t*		slot;
+	dict_table_t*		table;
 	ulint			i;
 
 	srv_sys = mem_alloc(sizeof(srv_sys_t));
@@ -806,6 +913,31 @@ srv_init(void)
 
 	UT_LIST_INIT(srv_sys->tasks);
 
+	/* create dummy table and index for old-style infimum and supremum */
+	table = dict_mem_table_create("SYS_DUMMY1",
+						DICT_HDR_SPACE, 1, FALSE);
+	dict_mem_table_add_col(table, "DUMMY", DATA_CHAR,
+					DATA_ENGLISH | DATA_NOT_NULL, 8, 0);
+
+	srv_sys->dummy_ind1 = dict_mem_index_create("SYS_DUMMY1",
+					"SYS_DUMMY1", DICT_HDR_SPACE, 0, 1);
+	dict_index_add_col(srv_sys->dummy_ind1,
+			dict_table_get_nth_col(table, 0), 0, 0);
+	srv_sys->dummy_ind1->table = table;
+	/* create dummy table and index for new-style infimum and supremum */
+	table = dict_mem_table_create("SYS_DUMMY2",
+						DICT_HDR_SPACE, 1, TRUE);
+	dict_mem_table_add_col(table, "DUMMY", DATA_CHAR,
+					DATA_ENGLISH | DATA_NOT_NULL, 8, 0);
+	srv_sys->dummy_ind2 = dict_mem_index_create("SYS_DUMMY2",
+					"SYS_DUMMY2", DICT_HDR_SPACE, 0, 1);
+	dict_index_add_col(srv_sys->dummy_ind2,
+			dict_table_get_nth_col(table, 0), 0, 0);
+	srv_sys->dummy_ind2->table = table;
+
+	/* avoid ut_ad(index->cached) in dict_index_get_n_unique_in_tree */
+	srv_sys->dummy_ind1->cached = srv_sys->dummy_ind2->cached = TRUE;
+
 	/* Init the server concurrency restriction data structures */
 
 	os_fast_mutex_init(&srv_conc_mutex);
@@ -813,7 +945,7 @@ srv_init(void)
 	UT_LIST_INIT(srv_conc_queue);
 
 	srv_conc_slots = mem_alloc(OS_THREAD_MAX_N * sizeof(srv_conc_slot_t));
-	
+
 	for (i = 0; i < OS_THREAD_MAX_N; i++) {
 		conc_slot = srv_conc_slots + i;
 		conc_slot->reserved = FALSE;
@@ -865,12 +997,6 @@ srv_conc_enter_innodb(
 	srv_conc_slot_t*	slot	  = NULL;
 	ulint			i;
 
-	if (srv_thread_concurrency >= 500) {
-		/* Disable the concurrency check */
-	
-		return;
-	}
-
 	/* If trx has 'free tickets' to enter the engine left, then use one
 	such ticket */
 
@@ -887,7 +1013,7 @@ retry:
 		fputs(
 "  InnoDB: Error: trying to declare trx to enter InnoDB, but\n"
 "InnoDB: it already is declared.\n", stderr);
-		trx_print(stderr, trx);
+		trx_print(stderr, trx, 0);
 		putc('\n', stderr);
 		os_fast_mutex_unlock(&srv_conc_mutex);
 
@@ -905,24 +1031,32 @@ retry:
 		return;
 	}
 
-	/* If the transaction is not holding resources, let it sleep for 50
-	milliseconds, and try again then */
+	/* If the transaction is not holding resources, 
+  let it sleep for SRV_THREAD_SLEEP_DELAY microseconds, and try again then */
  
 	if (!has_slept && !trx->has_search_latch
 	    && NULL == UT_LIST_GET_FIRST(trx->trx_locks)) {
 
-	        has_slept = TRUE; /* We let is sleep only once to avoid
+		has_slept = TRUE; /* We let it sleep only once to avoid
 				  starvation */
 
 		srv_conc_n_waiting_threads++;
 
 		os_fast_mutex_unlock(&srv_conc_mutex);
 
-		trx->op_info = (char*)"sleeping before joining InnoDB queue";
+		trx->op_info = "sleeping before joining InnoDB queue";
 
-		os_thread_sleep(50000);
+		/* Peter Zaitsev suggested that we take the sleep away
+		altogether. But the sleep may be good in pathological
+		situations of lots of thread switches. Simply put some
+		threads aside for a while to reduce the number of thread
+		switches. */
+    if (SRV_THREAD_SLEEP_DELAY > 0)
+    {
+      os_thread_sleep(SRV_THREAD_SLEEP_DELAY);
+    }
 
-		trx->op_info = (char*)"";
+		trx->op_info = "";
 
 		os_fast_mutex_lock(&srv_conc_mutex);
 
@@ -937,6 +1071,7 @@ retry:
 		slot = srv_conc_slots + i;
 
 		if (!slot->reserved) {
+
 			break;
 		}
 	}
@@ -974,11 +1109,11 @@ retry:
 	/* Go to wait for the event; when a thread leaves InnoDB it will
 	release this thread */
 
-	trx->op_info = (char*)"waiting in InnoDB queue";
+	trx->op_info = "waiting in InnoDB queue";
 
 	os_event_wait(slot->event);
 
-	trx->op_info = (char*)"";
+	trx->op_info = "";
 
 	os_fast_mutex_lock(&srv_conc_mutex);
 
@@ -1007,7 +1142,7 @@ srv_conc_force_enter_innodb(
 	trx_t*	trx)	/* in: transaction object associated with the
 			thread */
 {
-	if (srv_thread_concurrency >= 500) {
+	if (UNIV_LIKELY(!srv_thread_concurrency)) {
 	
 		return;
 	}
@@ -1016,7 +1151,7 @@ srv_conc_force_enter_innodb(
 
 	srv_conc_n_threads++;
 	trx->declared_to_be_inside_innodb = TRUE;
-	trx->n_tickets_to_enter_innodb = 0;
+	trx->n_tickets_to_enter_innodb = 1;
 
 	os_fast_mutex_unlock(&srv_conc_mutex);
 }
@@ -1033,7 +1168,7 @@ srv_conc_force_exit_innodb(
 {
 	srv_conc_slot_t*	slot	= NULL;
 
-	if (srv_thread_concurrency >= 500) {
+	if (UNIV_LIKELY(!srv_thread_concurrency)) {
 	
 		return;
 	}
@@ -1085,11 +1220,6 @@ srv_conc_exit_innodb(
 	trx_t*	trx)	/* in: transaction object associated with the
 			thread */
 {
-	if (srv_thread_concurrency >= 500) {
-	
-		return;
-	}
-
 	if (trx->n_tickets_to_enter_innodb > 0) {
 		/* We will pretend the thread is still inside InnoDB though it
 		now leaves the InnoDB engine. In this way we save
@@ -1131,9 +1261,19 @@ srv_normalize_init_values(void)
 
 	srv_log_buffer_size = srv_log_buffer_size / UNIV_PAGE_SIZE;
 
-	srv_pool_size = srv_pool_size / UNIV_PAGE_SIZE;
+	srv_pool_size = srv_pool_size / (UNIV_PAGE_SIZE / 1024);
+
+	srv_awe_window_size = srv_awe_window_size / UNIV_PAGE_SIZE;
 	
-	srv_lock_table_size = 20 * srv_pool_size;
+	if (srv_use_awe) {
+	        /* If we are using AWE we must save memory in the 32-bit
+		address space of the process, and cannot bind the lock
+		table size to the real buffer pool size. */
+
+	        srv_lock_table_size = 20 * srv_awe_window_size;
+	} else {
+	        srv_lock_table_size = 5 * srv_pool_size;
+	}
 
 	return(DB_SUCCESS);
 }
@@ -1169,6 +1309,7 @@ srv_boot(void)
 	return(DB_SUCCESS);
 }
 
+#ifndef UNIV_HOTBACKUP
 /*************************************************************************
 Reserves a slot in the thread table for the current MySQL OS thread.
 NOTE! The kernel mutex has to be reserved by the caller! */
@@ -1199,7 +1340,7 @@ srv_table_reserve_slot_for_mysql(void)
 "  InnoDB: There appear to be %lu MySQL threads currently waiting\n"
 "InnoDB: inside InnoDB, which is the upper limit. Cannot continue operation.\n"
 "InnoDB: We intentionally generate a seg fault to print a stack trace\n"
-"InnoDB: on Linux. But first we print a list of waiting threads.\n", i);
+"InnoDB: on Linux. But first we print a list of waiting threads.\n", (ulong) i);
 
 			for (i = 0; i < OS_THREAD_MAX_N; i++) {
 
@@ -1207,10 +1348,10 @@ srv_table_reserve_slot_for_mysql(void)
 
 			        fprintf(stderr,
 "Slot %lu: thread id %lu, type %lu, in use %lu, susp %lu, time %lu\n",
-				  i, os_thread_pf(slot->id),
-				  slot->type, slot->in_use,
-				  slot->suspended,
-			  (ulint)difftime(ut_time(), slot->suspend_time));
+				  (ulong) i, (ulong) os_thread_pf(slot->id),
+				  (ulong) slot->type, (ulong) slot->in_use,
+				  (ulong) slot->suspended,
+			  (ulong) difftime(ut_time(), slot->suspend_time));
 			}
 
 		        ut_error;
@@ -1227,6 +1368,7 @@ srv_table_reserve_slot_for_mysql(void)
 
 	return(slot);
 }
+#endif /* !UNIV_HOTBACKUP */
 
 /*******************************************************************
 Puts a MySQL OS thread to wait for a lock to be released. If an error
@@ -1241,13 +1383,19 @@ srv_suspend_mysql_thread(
 	que_thr_t*	thr)	/* in: query thread associated with the MySQL
 				OS thread */
 {
+#ifndef UNIV_HOTBACKUP
 	srv_slot_t*	slot;
 	os_event_t	event;
 	double		wait_time;
 	trx_t*		trx;
 	ibool		had_dict_lock			= FALSE;
 	ibool		was_declared_inside_innodb	= FALSE;
-	
+	ib_longlong	start_time			= 0;
+	ib_longlong	finish_time;
+	ulint		diff_time;
+	ulint		sec;
+	ulint		ms;
+
 #ifdef UNIV_SYNC_DEBUG
 	ut_ad(!mutex_own(&kernel_mutex));
 #endif /* UNIV_SYNC_DEBUG */
@@ -1290,6 +1438,13 @@ srv_suspend_mysql_thread(
 
 	slot->suspend_time = ut_time();
 
+	if (thr->lock_state == QUE_THR_LOCK_ROW) {
+		srv_n_lock_wait_count++;
+		srv_n_lock_wait_current_count++;
+
+		ut_usectime(&sec, &ms);
+		start_time = (ib_longlong)sec * 1000000 + ms;
+	}
 	/* Wake the lock timeout monitor thread, if it is suspended */
 
 	os_event_set(srv_lock_timeout_thread_event);
@@ -1340,7 +1495,20 @@ srv_suspend_mysql_thread(
 	slot->in_use = FALSE;
 
 	wait_time = ut_difftime(ut_time(), slot->suspend_time);
-	
+
+	if (thr->lock_state == QUE_THR_LOCK_ROW) {
+		ut_usectime(&sec, &ms);
+		finish_time = (ib_longlong)sec * 1000000 + ms;
+
+		diff_time = (ulint) (finish_time - start_time);
+  
+		srv_n_lock_wait_current_count--;
+		srv_n_lock_wait_time = srv_n_lock_wait_time + diff_time;
+		if (diff_time > srv_n_lock_max_wait_time) {
+			srv_n_lock_max_wait_time = diff_time;
+		}
+	}
+
 	if (trx->was_chosen_as_deadlock_victim) {
 
 		trx->error_state = DB_DEADLOCK;
@@ -1354,6 +1522,12 @@ srv_suspend_mysql_thread(
 
 	    	trx->error_state = DB_LOCK_WAIT_TIMEOUT;
 	}
+#else /* UNIV_HOTBACKUP */
+	/* This function depends on MySQL code that is not included in
+	InnoDB Hot Backup builds.  Besides, this function should never
+	be called in InnoDB Hot Backup. */
+	ut_error;
+#endif /* UNIV_HOTBACKUP */
 }
 
 /************************************************************************
@@ -1366,6 +1540,7 @@ srv_release_mysql_thread_if_suspended(
 	que_thr_t*	thr)	/* in: query thread associated with the
 				MySQL OS thread  */
 {
+#ifndef UNIV_HOTBACKUP
 	srv_slot_t*	slot;
 	ulint		i;
 	
@@ -1387,8 +1562,15 @@ srv_release_mysql_thread_if_suspended(
 	}
 
 	/* not found */
+#else /* UNIV_HOTBACKUP */
+	/* This function depends on MySQL code that is not included in
+	InnoDB Hot Backup builds.  Besides, this function should never
+	be called in InnoDB Hot Backup. */
+	ut_error;
+#endif /* UNIV_HOTBACKUP */
 }
 
+#ifndef UNIV_HOTBACKUP
 /**********************************************************************
 Refreshes the values used to calculate per-second averages. */
 static
@@ -1423,11 +1605,15 @@ Outputs to a file the output of the InnoDB Monitor. */
 void
 srv_printf_innodb_monitor(
 /*======================*/
-	FILE*	file)	/* in: output stream */
+	FILE*	file,		/* in: output stream */
+	ulint*	trx_start,	/* out: file position of the start of
+				the list of active transactions */
+	ulint*	trx_end)	/* out: file position of the end of
+				the list of active transactions */
 {
 	double	time_elapsed;
 	time_t	current_time;
-	ulint	n_reserved;
+	ulint   n_reserved;
 
 	mutex_enter(&srv_innodb_monitor_mutex);
 
@@ -1472,7 +1658,24 @@ srv_printf_innodb_monitor(
 
 	mutex_exit(&dict_foreign_err_mutex);
 
-	lock_print_info(file);
+	lock_print_info_summary(file);
+	if (trx_start) {
+		long	t = ftell(file);
+		if (t < 0) {
+			*trx_start = ULINT_UNDEFINED;
+		} else {
+			*trx_start = (ulint) t;
+		}
+	}
+	lock_print_info_all_transactions(file);
+	if (trx_end) {
+		long	t = ftell(file);
+		if (t < 0) {
+			*trx_end = ULINT_UNDEFINED;
+		} else {
+			*trx_end = (ulint) t;
+		}
+	}
 	fputs("--------\n"
 		"FILE I/O\n"
 		"--------\n", file);
@@ -1503,36 +1706,49 @@ srv_printf_innodb_monitor(
 		       "BUFFER POOL AND MEMORY\n"
 		"----------------------\n", file);
 	fprintf(file,
-	"Total memory allocated %lu; in additional pool allocated %lu\n",
+	"Total memory allocated " ULINTPF
+	"; in additional pool allocated " ULINTPF "\n",
 				ut_total_allocated_memory,
 				mem_pool_get_reserved(mem_comm_pool));
+
+	if (srv_use_awe) {
+		fprintf(file,
+	"In addition to that %lu MB of AWE memory allocated\n",
+		(ulong) (srv_pool_size / ((1024 * 1024) / UNIV_PAGE_SIZE)));
+	}
+	
 	buf_print_io(file);
 
 	fputs("--------------\n"
 		"ROW OPERATIONS\n"
 		"--------------\n", file);
 	fprintf(file, "%ld queries inside InnoDB, %lu queries in queue\n",
-			srv_conc_n_threads, srv_conc_n_waiting_threads);
+       		       (long) srv_conc_n_threads,
+		       (ulong) srv_conc_n_waiting_threads);
 
-	n_reserved = fil_space_get_n_reserved_extents(0);
-	if (n_reserved > 0) {
-		fprintf(file,
-	"%lu tablespace extents now reserved for B-tree split operations\n",
-						    n_reserved);
-	}
+	fprintf(file, "%lu read views open inside InnoDB\n",
+			UT_LIST_GET_LEN(trx_sys->view_list));
+
+        n_reserved = fil_space_get_n_reserved_extents(0);
+        if (n_reserved > 0) {
+                fprintf(file,
+        "%lu tablespace extents now reserved for B-tree split operations\n",
+                                                    (ulong) n_reserved);
+        }
 
 #ifdef UNIV_LINUX
 	fprintf(file, "Main thread process no. %lu, id %lu, state: %s\n",
-			srv_main_thread_process_no,
-			srv_main_thread_id,
-			srv_main_thread_op_info);
+		       (ulong) srv_main_thread_process_no,
+		       (ulong) srv_main_thread_id,
+		       srv_main_thread_op_info);
 #else
 	fprintf(file, "Main thread id %lu, state: %s\n",
-			srv_main_thread_id,
+			(ulong) srv_main_thread_id,
 			srv_main_thread_op_info);
 #endif
 	fprintf(file,
-	"Number of rows inserted %lu, updated %lu, deleted %lu, read %lu\n",
+	"Number of rows inserted " ULINTPF
+	", updated " ULINTPF ", deleted " ULINTPF ", read " ULINTPF "\n",
 			srv_n_rows_inserted, 
 			srv_n_rows_updated, 
 			srv_n_rows_deleted, 
@@ -1548,19 +1764,80 @@ srv_printf_innodb_monitor(
 			(srv_n_rows_read - srv_n_rows_read_old)
 						/ time_elapsed);
 
-	srv_n_rows_inserted_old = srv_n_rows_inserted;
+  srv_n_rows_inserted_old = srv_n_rows_inserted;
 	srv_n_rows_updated_old = srv_n_rows_updated;
 	srv_n_rows_deleted_old = srv_n_rows_deleted;
 	srv_n_rows_read_old = srv_n_rows_read;
 
-	fputs("----------------------------\n"
+  fputs("----------------------------\n"
 		       "END OF INNODB MONITOR OUTPUT\n"
 		"============================\n", file);
-
 	mutex_exit(&srv_innodb_monitor_mutex);
 	fflush(file);
 }
 
+/**********************************************************************
+Function to pass InnoDB status variables to MySQL */
+
+void
+srv_export_innodb_status(void)
+{
+
+        mutex_enter(&srv_innodb_monitor_mutex);
+        export_vars.innodb_data_pending_reads= os_n_pending_reads;
+        export_vars.innodb_data_pending_writes= os_n_pending_writes;
+        export_vars.innodb_data_pending_fsyncs= 
+                fil_n_pending_log_flushes + fil_n_pending_tablespace_flushes;
+        export_vars.innodb_data_fsyncs= os_n_fsyncs;
+        export_vars.innodb_data_read= srv_data_read;
+        export_vars.innodb_data_reads= os_n_file_reads;
+        export_vars.innodb_data_writes= os_n_file_writes;
+        export_vars.innodb_data_written= srv_data_written;
+        export_vars.innodb_buffer_pool_read_requests= buf_pool->n_page_gets;
+        export_vars.innodb_buffer_pool_write_requests= srv_buf_pool_write_requests;
+        export_vars.innodb_buffer_pool_wait_free= srv_buf_pool_wait_free;
+        export_vars.innodb_buffer_pool_pages_flushed= srv_buf_pool_flushed;
+        export_vars.innodb_buffer_pool_reads= srv_buf_pool_reads;
+        export_vars.innodb_buffer_pool_read_ahead_rnd= srv_read_ahead_rnd;
+        export_vars.innodb_buffer_pool_read_ahead_seq= srv_read_ahead_seq;
+        export_vars.innodb_buffer_pool_pages_data= UT_LIST_GET_LEN(buf_pool->LRU);
+        export_vars.innodb_buffer_pool_pages_dirty= UT_LIST_GET_LEN(buf_pool->flush_list);
+        export_vars.innodb_buffer_pool_pages_free= UT_LIST_GET_LEN(buf_pool->free);
+        export_vars.innodb_buffer_pool_pages_latched= buf_get_latched_pages_number();
+        export_vars.innodb_buffer_pool_pages_total= buf_pool->curr_size;
+        export_vars.innodb_buffer_pool_pages_misc= buf_pool->max_size -
+          UT_LIST_GET_LEN(buf_pool->LRU) - UT_LIST_GET_LEN(buf_pool->free);
+        export_vars.innodb_page_size= UNIV_PAGE_SIZE;
+        export_vars.innodb_log_waits= srv_log_waits;
+        export_vars.innodb_os_log_written= srv_os_log_written;
+        export_vars.innodb_os_log_fsyncs= fil_n_log_flushes;
+        export_vars.innodb_os_log_pending_fsyncs= fil_n_pending_log_flushes;
+        export_vars.innodb_os_log_pending_writes= srv_os_log_pending_writes;
+        export_vars.innodb_log_write_requests= srv_log_write_requests;
+        export_vars.innodb_log_writes= srv_log_writes;
+        export_vars.innodb_dblwr_pages_written= srv_dblwr_pages_written;
+        export_vars.innodb_dblwr_writes= srv_dblwr_writes;
+        export_vars.innodb_pages_created= buf_pool->n_pages_created;
+        export_vars.innodb_pages_read= buf_pool->n_pages_read;
+        export_vars.innodb_pages_written= buf_pool->n_pages_written;
+        export_vars.innodb_row_lock_waits= srv_n_lock_wait_count;
+        export_vars.innodb_row_lock_current_waits= srv_n_lock_wait_current_count;
+        export_vars.innodb_row_lock_time= srv_n_lock_wait_time / 1000;
+	if (srv_n_lock_wait_count > 0) {
+		export_vars.innodb_row_lock_time_avg = (ulint)
+			(srv_n_lock_wait_time / 1000 / srv_n_lock_wait_count);
+	} else {
+		export_vars.innodb_row_lock_time_avg = 0;
+	}
+        export_vars.innodb_row_lock_time_max= srv_n_lock_max_wait_time / 1000;
+        export_vars.innodb_rows_read= srv_n_rows_read;
+        export_vars.innodb_rows_inserted= srv_n_rows_inserted;
+        export_vars.innodb_rows_updated= srv_n_rows_updated;
+        export_vars.innodb_rows_deleted= srv_n_rows_deleted;
+        mutex_exit(&srv_innodb_monitor_mutex);
+
+}
+
 /*************************************************************************
 A thread which wakes up threads whose lock wait may have lasted too long.
 This also prints the info output by various InnoDB monitors. */
@@ -1581,6 +1858,7 @@ srv_lock_timeout_and_monitor_thread(
 	double		time_elapsed;
 	time_t          current_time;
 	time_t		last_table_monitor_time;
+	time_t		last_tablespace_monitor_time;
 	time_t		last_monitor_time;
 	ibool		some_waits;
 	double		wait_time;
@@ -1593,6 +1871,7 @@ srv_lock_timeout_and_monitor_thread(
 	UT_NOT_USED(arg);
 	srv_last_monitor_time = time(NULL);
 	last_table_monitor_time = time(NULL);
+	last_tablespace_monitor_time = time(NULL);
 	last_monitor_time = time(NULL);
 loop:
 	srv_lock_timeout_and_monitor_active = TRUE;
@@ -1602,12 +1881,6 @@ loop:
 
 	os_thread_sleep(1000000);
 
-	/* In case mutex_exit is not a memory barrier, it is
-	theoretically possible some threads are left waiting though
-	the semaphore is already released. Wake up those threads: */
-	
-	sync_arr_wake_threads_if_sema_free();
-
 	current_time = time(NULL);
 
 	time_elapsed = difftime(current_time, last_monitor_time);
@@ -1616,21 +1889,21 @@ loop:
 	    last_monitor_time = time(NULL);
 
 	    if (srv_print_innodb_monitor) {
-		srv_printf_innodb_monitor(stderr);
+		srv_printf_innodb_monitor(stderr, NULL, NULL);
 	    }
 
 	    if (srv_innodb_status) {
 		mutex_enter(&srv_monitor_file_mutex);
 		rewind(srv_monitor_file);
-		srv_printf_innodb_monitor(srv_monitor_file);
+		srv_printf_innodb_monitor(srv_monitor_file, NULL, NULL);
 		os_file_set_eof(srv_monitor_file);
 		mutex_exit(&srv_monitor_file_mutex);
 	    }
 
 	    if (srv_print_innodb_tablespace_monitor
-		&& difftime(current_time, last_table_monitor_time) > 60) {
+		&& difftime(current_time, last_tablespace_monitor_time) > 60) {
 
-		last_table_monitor_time = time(NULL);	
+		last_tablespace_monitor_time = time(NULL);	
 
 		fputs("================================================\n",
 			stderr);
@@ -1766,6 +2039,10 @@ srv_error_monitor_thread(
 {
 	/* number of successive fatal timeouts observed */
 	ulint	fatal_cnt	= 0;
+	dulint	old_lsn;
+	dulint	new_lsn;
+
+	old_lsn = srv_start_lsn;
 
 #ifdef UNIV_DEBUG_THREAD_CREATION
 	fprintf(stderr, "Error monitor thread starts, id %lu\n",
@@ -1774,7 +2051,24 @@ srv_error_monitor_thread(
 loop:
 	srv_error_monitor_active = TRUE;
 
-	os_thread_sleep(2000000);
+	/* Try to track a strange bug reported by Harald Fuchs and others,
+	where the lsn seems to decrease at times */
+
+	new_lsn = log_get_lsn();
+
+	if (ut_dulint_cmp(new_lsn, old_lsn) < 0) {
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+"  InnoDB: Error: old log sequence number %lu %lu was greater\n"
+"InnoDB: than the new log sequence number %lu %lu!\n"
+"InnoDB: Please send a bug report to mysql@lists.mysql.com\n",
+		(ulong) ut_dulint_get_high(old_lsn),
+		(ulong) ut_dulint_get_low(old_lsn),
+		(ulong) ut_dulint_get_high(new_lsn),
+		(ulong) ut_dulint_get_low(new_lsn));
+	}
+
+	old_lsn = new_lsn;
 
 	if (difftime(time(NULL), srv_last_monitor_time) > 60) {
 		/* We referesh InnoDB Monitor values so that averages are
@@ -1783,9 +2077,15 @@ loop:
 		srv_refresh_innodb_monitor_stats();
 	}
 
+	/* In case mutex_exit is not a memory barrier, it is
+	theoretically possible some threads are left waiting though
+	the semaphore is already released. Wake up those threads: */
+	
+	sync_arr_wake_threads_if_sema_free();
+
 	if (sync_array_print_long_waits()) {
 		fatal_cnt++;
-		if (fatal_cnt > 5) {
+		if (fatal_cnt > 10) {
 
 			fprintf(stderr,
 "InnoDB: Error: semaphore wait has lasted > %lu seconds\n"
@@ -1803,7 +2103,9 @@ loop:
 
 	fflush(stderr);
 
-	if (srv_shutdown_state < SRV_SHUTDOWN_LAST_PHASE) {
+	os_thread_sleep(1000000);
+
+	if (srv_shutdown_state < SRV_SHUTDOWN_CLEANUP) {
 
 		goto loop;
 	}
@@ -1913,7 +2215,7 @@ loop:
 	/* ---- When there is database activity by users, we cycle in this
 	loop */
 
-	srv_main_thread_op_info = (char*) "reserving kernel mutex";
+	srv_main_thread_op_info = "reserving kernel mutex";
 
 	n_ios_very_old = log_sys->n_log_ios + buf_pool->n_pages_read
 						+ buf_pool->n_pages_written;
@@ -1937,7 +2239,7 @@ loop:
 	for (i = 0; i < 10; i++) {
 		n_ios_old = log_sys->n_log_ios + buf_pool->n_pages_read
 						+ buf_pool->n_pages_written;
-		srv_main_thread_op_info = (char*)"sleeping";
+		srv_main_thread_op_info = "sleeping";
 		
 		if (!skip_sleep) {
 
@@ -1950,12 +2252,11 @@ loop:
 		can drop tables lazily after there no longer are SELECT
 		queries to them. */
 
-		srv_main_thread_op_info =
-					(char*)"doing background drop tables";
+		srv_main_thread_op_info = "doing background drop tables";
 
 		row_drop_tables_for_mysql_in_background();
 
-		srv_main_thread_op_info = (char*)"";
+		srv_main_thread_op_info = "";
 
 		if (srv_fast_shutdown && srv_shutdown_state > 0) {
 
@@ -1966,9 +2267,12 @@ loop:
 		is issued or the we have specified in my.cnf no flush
 		at transaction commit */
 
-		srv_main_thread_op_info = (char*)"flushing log";
+		srv_main_thread_op_info = "flushing log";
 		log_buffer_flush_to_disk();
 
+		srv_main_thread_op_info = "making checkpoint";
+		log_free_check();
+
 		/* If there were less than 5 i/os during the
 		one second sleep, we assume that there is free
 		disk i/o capacity available, and it makes sense to
@@ -1979,11 +2283,10 @@ loop:
 		n_ios = log_sys->n_log_ios + buf_pool->n_pages_read
 						+ buf_pool->n_pages_written;
 		if (n_pend_ios < 3 && (n_ios - n_ios_old < 5)) {
-			srv_main_thread_op_info =
-					(char*)"doing insert buffer merge";
+			srv_main_thread_op_info = "doing insert buffer merge";
 			ibuf_contract_for_n_pages(TRUE, 5);
 
-			srv_main_thread_op_info = (char*)"flushing log";
+			srv_main_thread_op_info = "flushing log";
 
 			log_buffer_flush_to_disk();
 		}
@@ -2031,20 +2334,20 @@ loop:
 						+ buf_pool->n_pages_written;
 	if (n_pend_ios < 3 && (n_ios - n_ios_very_old < 200)) {
 
-		srv_main_thread_op_info = (char*) "flushing buffer pool pages";
+		srv_main_thread_op_info = "flushing buffer pool pages";
 		buf_flush_batch(BUF_FLUSH_LIST, 100, ut_dulint_max);
 
-		srv_main_thread_op_info = (char*) "flushing log";
+		srv_main_thread_op_info = "flushing log";
 		log_buffer_flush_to_disk();
 	}
 
 	/* We run a batch of insert buffer merge every 10 seconds,
 	even if the server were active */
 
-	srv_main_thread_op_info = (char*)"doing insert buffer merge";
+	srv_main_thread_op_info = "doing insert buffer merge";
 	ibuf_contract_for_n_pages(TRUE, 5);
 
-	srv_main_thread_op_info = (char*)"flushing log";
+	srv_main_thread_op_info = "flushing log";
 	log_buffer_flush_to_disk();
 
 	/* We run a full purge every 10 seconds, even if the server
@@ -2061,20 +2364,20 @@ loop:
 			goto background_loop;
 		}
 
-		srv_main_thread_op_info = (char*)"purging";
+		srv_main_thread_op_info = "purging";
 		n_pages_purged = trx_purge();
 
 		current_time = time(NULL);
 
 		if (difftime(current_time, last_flush_time) > 1) {
-			srv_main_thread_op_info = (char*) "flushing log";
+			srv_main_thread_op_info = "flushing log";
 
 		        log_buffer_flush_to_disk();
 			last_flush_time = current_time;
 		}
 	}
 	
-	srv_main_thread_op_info = (char*)"flushing buffer pool pages";
+	srv_main_thread_op_info = "flushing buffer pool pages";
 
 	/* Flush a few oldest pages to make a new checkpoint younger */
 
@@ -2095,13 +2398,13 @@ loop:
 							ut_dulint_max);
 	}
 
-	srv_main_thread_op_info = (char*)"making checkpoint";
+	srv_main_thread_op_info = "making checkpoint";
 
 	/* Make a new checkpoint about once in 10 seconds */
 
 	log_checkpoint(TRUE, FALSE);
 
-	srv_main_thread_op_info = (char*)"reserving kernel mutex";
+	srv_main_thread_op_info = "reserving kernel mutex";
 
 	mutex_enter(&kernel_mutex);
 	
@@ -2120,12 +2423,13 @@ loop:
 	/*****************************************************************/
 background_loop:
 	/* ---- In this loop we run background operations when the server
-	is quiet from user activity */
+	is quiet from user activity. Also in the case of a shutdown, we
+	loop here, flushing the buffer pool to the data files. */
 
 	/* The server has been quiet for a while: start running background
 	operations */
 		
-	srv_main_thread_op_info = (char*)"doing background drop tables";
+	srv_main_thread_op_info = "doing background drop tables";
 
 	n_tables_to_drop = row_drop_tables_for_mysql_in_background();
 
@@ -2138,7 +2442,7 @@ background_loop:
 		os_thread_sleep(100000);
 	}
  
-	srv_main_thread_op_info = (char*)"purging";
+	srv_main_thread_op_info = "purging";
 
 	/* Run a full purge */
 	
@@ -2152,20 +2456,20 @@ background_loop:
 			break;
 		}
 
-		srv_main_thread_op_info = (char*)"purging";
+		srv_main_thread_op_info = "purging";
 		n_pages_purged = trx_purge();
 
 		current_time = time(NULL);
 
 		if (difftime(current_time, last_flush_time) > 1) {
-			srv_main_thread_op_info = (char*) "flushing log";
+			srv_main_thread_op_info = "flushing log";
 
 		        log_buffer_flush_to_disk();
 			last_flush_time = current_time;
 		}
 	}
 
-	srv_main_thread_op_info = (char*)"reserving kernel mutex";
+	srv_main_thread_op_info = "reserving kernel mutex";
 
 	mutex_enter(&kernel_mutex);
 	if (srv_activity_count != old_activity_count) {
@@ -2174,7 +2478,7 @@ background_loop:
 	}
 	mutex_exit(&kernel_mutex);
 
-	srv_main_thread_op_info = (char*)"doing insert buffer merge";
+	srv_main_thread_op_info = "doing insert buffer merge";
 
 	if (srv_fast_shutdown && srv_shutdown_state > 0) {
 	        n_bytes_merged = 0;
@@ -2182,7 +2486,7 @@ background_loop:
 	        n_bytes_merged = ibuf_contract_for_n_pages(TRUE, 20);
 	}
 
-	srv_main_thread_op_info = (char*)"reserving kernel mutex";
+	srv_main_thread_op_info = "reserving kernel mutex";
 
 	mutex_enter(&kernel_mutex);
 	if (srv_activity_count != old_activity_count) {
@@ -2192,10 +2496,19 @@ background_loop:
 	mutex_exit(&kernel_mutex);
 	
 flush_loop:
-	srv_main_thread_op_info = (char*)"flushing buffer pool pages";
-	n_pages_flushed = buf_flush_batch(BUF_FLUSH_LIST, 100, ut_dulint_max);
+	srv_main_thread_op_info = "flushing buffer pool pages";
 
-	srv_main_thread_op_info = (char*)"reserving kernel mutex";
+	if (srv_fast_shutdown < 2) {
+		n_pages_flushed =
+			buf_flush_batch(BUF_FLUSH_LIST, 100, ut_dulint_max);
+	} else {
+		/* In the fastest shutdown we do not flush the buffer pool
+		to data files: we set n_pages_flushed to 0 artificially. */
+
+		n_pages_flushed = 0;
+	}
+
+	srv_main_thread_op_info = "reserving kernel mutex";
 
 	mutex_enter(&kernel_mutex);
 	if (srv_activity_count != old_activity_count) {
@@ -2204,15 +2517,14 @@ flush_loop:
 	}
 	mutex_exit(&kernel_mutex);
 	
-	srv_main_thread_op_info =
-			(char*) "waiting for buffer pool flush to end";
+	srv_main_thread_op_info = "waiting for buffer pool flush to end";
 	buf_flush_wait_batch_end(BUF_FLUSH_LIST);
 
-	srv_main_thread_op_info = (char*) "flushing log";
+	srv_main_thread_op_info = "flushing log";
 
 	log_buffer_flush_to_disk();
 
-	srv_main_thread_op_info = (char*)"making checkpoint";
+	srv_main_thread_op_info = "making checkpoint";
 
 	log_checkpoint(TRUE, FALSE);
 
@@ -2224,7 +2536,7 @@ flush_loop:
 		goto flush_loop;
 	}
 
-	srv_main_thread_op_info = (char*)"reserving kernel mutex";
+	srv_main_thread_op_info = "reserving kernel mutex";
 
 	mutex_enter(&kernel_mutex);
 	if (srv_activity_count != old_activity_count) {
@@ -2232,11 +2544,12 @@ flush_loop:
 		goto loop;
 	}
 	mutex_exit(&kernel_mutex);
-
-	srv_main_thread_op_info =
-				(char*)"archiving log (if log archive is on)";
+/*
+	srv_main_thread_op_info = "archiving log (if log archive is on)";
 	
 	log_archive_do(FALSE, &n_bytes_archived);
+*/
+	n_bytes_archived = 0;
 
 	/* Keep looping in the background loop if still work to do */
 
@@ -2246,7 +2559,10 @@ flush_loop:
 
 			/* If we are doing a fast shutdown (= the default)
 			we do not do purge or insert buffer merge. But we
-			flush the buffer pool completely to disk. */
+			flush the buffer pool completely to disk.
+			In a 'very fast' shutdown we do not flush the buffer
+			pool to data files: we have set n_pages_flushed to
+			0 artificially. */
 
 			goto background_loop;
 		}
@@ -2263,7 +2579,7 @@ flush_loop:
 	master thread to wait for more server activity */
 	
 suspend_thread:
-	srv_main_thread_op_info = (char*)"suspending";
+	srv_main_thread_op_info = "suspending";
 
 	mutex_enter(&kernel_mutex);
 
@@ -2277,7 +2593,7 @@ suspend_thread:
 
 	mutex_exit(&kernel_mutex);
 
-	srv_main_thread_op_info = (char*)"waiting for server activity";
+	srv_main_thread_op_info = "waiting for server activity";
 
 	os_event_wait(event);
 
@@ -2301,8 +2617,9 @@ suspend_thread:
 	os_thread_exit(NULL);
 
 #ifndef __WIN__
-        return(NULL);
+        return(NULL);				/* Not reached */
 #else
 	return(0);
 #endif
 }
+#endif /* !UNIV_HOTBACKUP */
