@@ -1,9 +1,8 @@
-/* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2000-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -234,6 +233,7 @@ static int berkeley_close_connection(THD *thd)
   return 0;
 }
 
+
 bool berkeley_flush_logs()
 {
   int error;
@@ -440,6 +440,15 @@ ulong ha_berkeley::index_flags(uint idx, uint part, bool all_parts) const
 }
 
 
+void ha_berkeley::get_auto_primary_key(byte *to)
+{
+  pthread_mutex_lock(&share->mutex);
+  share->auto_ident++;
+  int5store(to,share->auto_ident);
+  pthread_mutex_unlock(&share->mutex);
+}
+
+
 static int
 berkeley_cmp_hidden_key(DB* file, const DBT *new_key, const DBT *saved_key)
 {
@@ -458,7 +467,7 @@ berkeley_cmp_packed_key(DB *file, const DBT *new_key, const DBT *saved_key)
   KEY_PART_INFO *key_part= key->key_part, *end=key_part+key->key_parts;
   uint key_length=new_key->size;
 
-  DBUG_DUMP("key_in_index", saved_key_ptr, saved_key->size);
+  DBUG_DUMP("key_in_index", (uchar *)saved_key_ptr, saved_key->size);
   for (; key_part != end && (int) key_length > 0; key_part++)
   {
     int cmp;
@@ -798,7 +807,7 @@ int ha_berkeley::pack_row(DBT *row, const byte *record, bool new_row)
     ptr+=BDB_HIDDEN_PRIMARY_KEY_LENGTH;
   }
   row->data=rec_buff;
-  row->size= (size_t) (ptr - rec_buff);
+  row->size= (u_int32_t) (ptr - rec_buff);
   return 0;
 }
 
@@ -893,8 +902,8 @@ DBT *ha_berkeley::create_key(DBT *key, uint keynr, char *buff,
 				   key_part->length);
     key_length-=key_part->length;
   }
-  key->size= (buff  - (char*) key->data);
-  DBUG_DUMP("key",(char*) key->data, key->size);
+  key->size= (u_int32_t) (buff  - (char*) key->data);
+  DBUG_DUMP("key",(uchar*) key->data, key->size);
   DBUG_RETURN(key);
 }
 
@@ -937,8 +946,8 @@ DBT *ha_berkeley::pack_key(DBT *key, uint keynr, char *buff,
     key_ptr+=key_part->store_length;
     key_length-=key_part->store_length;
   }
-  key->size= (buff  - (char*) key->data);
-  DBUG_DUMP("key",(char*) key->data, key->size);
+  key->size= (u_int32_t) (buff  - (char*) key->data);
+  DBUG_DUMP("key",(uchar*) key->data, key->size);
   DBUG_RETURN(key);
 }
 
@@ -953,7 +962,10 @@ int ha_berkeley::write_row(byte * record)
   if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_INSERT)
     table->timestamp_field->set_time();
   if (table->next_number_field && record == table->record[0])
-    update_auto_increment();
+  {
+    if ((error= update_auto_increment()))
+      DBUG_RETURN(error);
+  }
   if ((error=pack_row(&row, record,1)))
     DBUG_RETURN(error); /* purecov: inspected */
 
@@ -1693,7 +1705,7 @@ DBT *ha_berkeley::get_pos(DBT *to, byte *pos)
       pos+=key_part->field->packed_col_length((char*) pos,key_part->length);
     to->size= (uint) (pos- (byte*) to->data);
   }
-  DBUG_DUMP("key", (char*) to->data, to->size);
+  DBUG_DUMP("key", (uchar*) to->data, to->size);
   return to;
 }
 
@@ -1753,7 +1765,7 @@ void ha_berkeley::position(const byte *record)
 }
 
 
-void ha_berkeley::info(uint flag)
+int ha_berkeley::info(uint flag)
 {
   DBUG_ENTER("ha_berkeley::info");
   if (flag & HA_STATUS_VARIABLE)
@@ -1773,7 +1785,7 @@ void ha_berkeley::info(uint flag)
   /* Don't return key if we got an error for the internal primary key */
   if (flag & HA_STATUS_ERRKEY && last_dup_key < table->s->keys)
     errkey= last_dup_key;
-  DBUG_VOID_RETURN;
+  DBUG_RETURN(0);
 }
 
 
@@ -1853,8 +1865,8 @@ int ha_berkeley::external_lock(THD *thd, int lock_type)
                            OPTION_TABLE_LOCK)) && !trx->all)
       {
 	/* We have to start a master transaction */
-	DBUG_PRINT("trans",("starting transaction all:  options: 0x%lx",
-                            (ulong) thd->options));
+	DBUG_PRINT("trans",("starting transaction all:  options: 0x%llx",
+                            thd->options));
         if ((error=txn_begin(db_env, 0, &trx->all, 0)))
 	{
           trx->bdb_lock_count--;        // We didn't get the lock
@@ -2634,7 +2646,11 @@ ha_rows ha_berkeley::estimate_rows_upper_bound()
 int ha_berkeley::cmp_ref(const byte *ref1, const byte *ref2)
 {
   if (hidden_primary_key)
-    return memcmp(ref1, ref2, BDB_HIDDEN_PRIMARY_KEY_LENGTH);
+  {
+    ulonglong a=uint5korr((char*) ref1);
+    ulonglong b=uint5korr((char*) ref2);
+    return  a < b ? -1 : (a > b ? 1 : 0);
+  }
 
   int result;
   Field *field;
