@@ -1,9 +1,8 @@
-/* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2000-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,16 +19,60 @@
 #include "mysql_priv.h"
 
 static int rr_quick(READ_RECORD *info);
-static int rr_sequential(READ_RECORD *info);
+int rr_sequential(READ_RECORD *info);
 static int rr_from_tempfile(READ_RECORD *info);
 static int rr_unpack_from_tempfile(READ_RECORD *info);
 static int rr_unpack_from_buffer(READ_RECORD *info);
 static int rr_from_pointers(READ_RECORD *info);
 static int rr_from_cache(READ_RECORD *info);
-static int init_rr_cache(READ_RECORD *info);
+static int init_rr_cache(THD *thd, READ_RECORD *info);
 static int rr_cmp(uchar *a,uchar *b);
+static int rr_index_first(READ_RECORD *info);
+static int rr_index(READ_RECORD *info);
 
-	/* init struct for read with info->read_record */
+
+/*
+  Initialize READ_RECORD structure to perform full index scan
+
+  SYNOPSIS 
+    init_read_record_idx()
+      info         READ_RECORD structure to initialize.
+      thd          Thread handle
+      table        Table to be accessed 
+      print_error  If true, call table->file->print_error() if an error
+                   occurs (except for end-of-records error)
+      idx          index to scan
+  
+  DESCRIPTION
+    Initialize READ_RECORD structure to perform full index scan (in forward
+    direction) using read_record.read_record() interface.
+    
+    This function has been added at late stage and is used only by
+    UPDATE/DELETE. Other statements perform index scans using
+    join_read_first/next functions.
+*/
+
+void init_read_record_idx(READ_RECORD *info, THD *thd, TABLE *table,
+                          bool print_error, uint idx)
+{
+  bzero((char*) info,sizeof(*info));
+  info->table= table;
+  info->file=  table->file;
+  info->record= table->record[0];
+  info->print_error= print_error;
+
+  table->status=0;			/* And it's always found */
+  if (!table->file->inited)
+  {
+    table->file->ha_index_init(idx);
+    table->file->extra(HA_EXTRA_RETRIEVE_PRIMARY_KEY);
+  }
+  /* read_record will be changed to rr_index in rr_index_first */
+  info->read_record= rr_index_first;
+}
+
+
+/* init struct for read with info->read_record */
 
 void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
 		      SQL_SELECT *select,
@@ -84,14 +127,15 @@ void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
 	!(table->file->table_flags() & HA_FAST_KEY_READ) &&
 	(table->db_stat & HA_READ_ONLY ||
 	 table->reginfo.lock_type <= TL_READ_NO_INSERT) &&
-	(ulonglong) table->reclength*(table->file->records+
-				      table->file->deleted) >
+	(ulonglong) table->s->reclength* (table->file->records+
+                                          table->file->deleted) >
 	(ulonglong) MIN_FILE_LENGTH_TO_USE_ROW_CACHE &&
-	info->io_cache->end_of_file/info->ref_length*table->reclength >
+	info->io_cache->end_of_file/info->ref_length * table->s->reclength >
 	(my_off_t) MIN_ROWS_TO_USE_TABLE_CACHE &&
-	!table->blob_fields)
+	!table->s->blob_fields &&
+        info->ref_length <= MAX_REFLENGTH)
     {
-      if (! init_rr_cache(info))
+      if (! init_rr_cache(thd, info))
       {
 	DBUG_PRINT("info",("using rr_from_cache"));
 	info->read_record=rr_from_cache;
@@ -101,9 +145,6 @@ void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
   else if (select && select->quick)
   {
     DBUG_PRINT("info",("using rr_quick"));
-
-    if (!table->file->inited)
-      table->file->ha_index_init(select->quick->index);
     info->read_record=rr_quick;
   }
   else if (table->sort.record_pointers)
@@ -125,16 +166,24 @@ void init_read_record(READ_RECORD *info,THD *thd, TABLE *table,
     if (!table->no_cache &&
 	(use_record_cache > 0 ||
 	 (int) table->reginfo.lock_type <= (int) TL_READ_HIGH_PRIORITY ||
-	 !(table->db_options_in_use & HA_OPTION_PACK_RECORD) ||
+	 !(table->s->db_options_in_use & HA_OPTION_PACK_RECORD) ||
 	 (use_record_cache < 0 &&
 	  !(table->file->table_flags() & HA_NOT_DELETE_WITH_CACHE))))
       VOID(table->file->extra_opt(HA_EXTRA_CACHE,
 				  thd->variables.read_buff_size));
   }
+  /* Condition pushdown to storage engine */
+  if (thd->variables.engine_condition_pushdown && 
+      select && select->cond && 
+      (select->cond->used_tables() & table->map) &&
+      !table->file->pushed_cond)
+    table->file->cond_push(select->cond);
+
   DBUG_VOID_RETURN;
 } /* init_read_record */
 
 
+
 void end_read_record(READ_RECORD *info)
 {                   /* free cache if used */
   if (info->cache)
@@ -144,7 +193,7 @@ void end_read_record(READ_RECORD *info)
   }
   if (info->table)
   {
-    filesort_free_buffers(info->table);
+    filesort_free_buffers(info->table,0);
     (void) info->file->extra(HA_EXTRA_NO_CACHE);
     if (info->read_record != rr_quick) // otherwise quick_range does it
       (void) info->file->ha_index_or_rnd_end();
@@ -152,6 +201,21 @@ void end_read_record(READ_RECORD *info)
   }
 }
 
+static int rr_handle_error(READ_RECORD *info, int error)
+{
+  if (error == HA_ERR_END_OF_FILE)
+    error= -1;
+  else
+  {
+    if (info->print_error)
+      info->table->file->print_error(error, MYF(0));
+    if (error < 0)                            // Fix negative BDB errno
+      error= 1;
+  }
+  return error;
+}
+
+
 	/* Read a record from head-database */
 
 static int rr_quick(READ_RECORD *info)
@@ -166,15 +230,7 @@ static int rr_quick(READ_RECORD *info)
     }
     if (tmp != HA_ERR_RECORD_DELETED)
     {
-      if (tmp == HA_ERR_END_OF_FILE)
-        tmp= -1;
-      else
-      {
-        if (info->print_error)
-          info->file->print_error(tmp,MYF(0));
-        if (tmp < 0)				// Fix negative BDB errno
-          tmp=1;
-      }
+      tmp= rr_handle_error(info, tmp);
       break;
     }
   }
@@ -182,27 +238,74 @@ static int rr_quick(READ_RECORD *info)
 }
 
 
-static int rr_sequential(READ_RECORD *info)
+/*
+  Reads first row in an index scan
+
+  SYNOPSIS
+    rr_index_first()
+    info  	Scan info
+  
+  RETURN
+    0   Ok
+    -1  End of records 
+    1   Error   
+*/
+
+
+static int rr_index_first(READ_RECORD *info)
+{
+  int tmp= info->file->index_first(info->record);
+  info->read_record= rr_index;
+  if (tmp)
+    tmp= rr_handle_error(info, tmp);
+  return tmp;
+}
+
+
+/*
+  Reads index sequentially after first row
+
+  SYNOPSIS
+    rr_index()
+      info  Scan info
+  
+  DESCRIPTION
+    Read the next index record (in forward direction) and translate return
+    value.
+    
+  RETURN
+    0   Ok
+    -1  End of records 
+    1   Error   
+*/
+
+
+static int rr_index(READ_RECORD *info)
+{
+  int tmp= info->file->index_next(info->record);
+  if (tmp)
+    tmp= rr_handle_error(info, tmp);
+  return tmp;
+}
+
+
+int rr_sequential(READ_RECORD *info)
 {
   int tmp;
   while ((tmp=info->file->rnd_next(info->record)))
   {
     if (info->thd->killed)
     {
-      my_error(ER_SERVER_SHUTDOWN,MYF(0));
+      info->thd->send_kill_message();
       return 1;
     }
+    /*
+      rnd_next can return RECORD_DELETED for MyISAM when one thread is
+      reading and another deleting without locks.
+    */
     if (tmp != HA_ERR_RECORD_DELETED)
     {
-      if (tmp == HA_ERR_END_OF_FILE)
-	tmp= -1;
-      else
-      {
-	if (info->print_error)
-	  info->table->file->print_error(tmp,MYF(0));
-	if (tmp < 0)				// Fix negative BDB errno
-	  tmp=1;
-      }
+      tmp= rr_handle_error(info, tmp);
       break;
     }
   }
@@ -213,23 +316,18 @@ static int rr_sequential(READ_RECORD *info)
 static int rr_from_tempfile(READ_RECORD *info)
 {
   int tmp;
-tryNext:
-  if (my_b_read(info->io_cache,info->ref_pos,info->ref_length))
-    return -1;					/* End of file */
-  if ((tmp=info->file->rnd_pos(info->record,info->ref_pos)))
+  for (;;)
   {
-    if (tmp == HA_ERR_END_OF_FILE)
-      tmp= -1;
-    else if (tmp == HA_ERR_RECORD_DELETED ||
-	     (tmp == HA_ERR_KEY_NOT_FOUND && info->ignore_not_found_rows))
-      goto tryNext;
-    else
-    {
-      if (info->print_error)
-	info->file->print_error(tmp,MYF(0));
-      if (tmp < 0)				// Fix negative BDB errno
-	tmp=1;
-    }
+    if (my_b_read(info->io_cache,info->ref_pos,info->ref_length))
+      return -1;					/* End of file */
+    if (!(tmp=info->file->rnd_pos(info->record,info->ref_pos)))
+      break;
+    /* The following is extremely unlikely to happen */
+    if (tmp == HA_ERR_RECORD_DELETED ||
+        (tmp == HA_ERR_KEY_NOT_FOUND && info->ignore_not_found_rows))
+      continue;
+    tmp= rr_handle_error(info, tmp);
+    break;
   }
   return tmp;
 } /* rr_from_tempfile */
@@ -267,26 +365,23 @@ static int rr_from_pointers(READ_RECORD *info)
 {
   int tmp;
   byte *cache_pos;
-tryNext:
-  if (info->cache_pos == info->cache_end)
-    return -1;					/* End of file */
-  cache_pos=info->cache_pos;
-  info->cache_pos+=info->ref_length;
 
-  if ((tmp=info->file->rnd_pos(info->record,cache_pos)))
+  for (;;)
   {
-    if (tmp == HA_ERR_END_OF_FILE)
-      tmp= -1;
-    else if (tmp == HA_ERR_RECORD_DELETED ||
-	     (tmp == HA_ERR_KEY_NOT_FOUND && info->ignore_not_found_rows))
-      goto tryNext;
-    else
-    {
-      if (info->print_error)
-	info->file->print_error(tmp,MYF(0));
-      if (tmp < 0)				// Fix negative BDB errno
-	tmp=1;
-    }
+    if (info->cache_pos == info->cache_end)
+      return -1;					/* End of file */
+    cache_pos= info->cache_pos;
+    info->cache_pos+= info->ref_length;
+
+    if (!(tmp=info->file->rnd_pos(info->record,cache_pos)))
+      break;
+
+    /* The following is extremely unlikely to happen */
+    if (tmp == HA_ERR_RECORD_DELETED ||
+        (tmp == HA_ERR_KEY_NOT_FOUND && info->ignore_not_found_rows))
+      continue;
+    tmp= rr_handle_error(info, tmp);
+    break;
   }
   return tmp;
 }
@@ -321,23 +416,21 @@ static int rr_unpack_from_buffer(READ_RECORD *info)
 }
 	/* cacheing of records from a database */
 
-static int init_rr_cache(READ_RECORD *info)
+static int init_rr_cache(THD *thd, READ_RECORD *info)
 {
   uint rec_cache_size;
-  THD *thd= current_thd;
-
   DBUG_ENTER("init_rr_cache");
 
-  info->struct_length=3+MAX_REFLENGTH;
-  info->reclength=ALIGN_SIZE(info->table->reclength+1);
+  info->struct_length= 3+MAX_REFLENGTH;
+  info->reclength= ALIGN_SIZE(info->table->s->reclength+1);
   if (info->reclength < info->struct_length)
-    info->reclength=ALIGN_SIZE(info->struct_length);
+    info->reclength= ALIGN_SIZE(info->struct_length);
 
-  info->error_offset=info->table->reclength;
-  info->cache_records= thd->variables.read_rnd_buff_size /
-    (info->reclength+info->struct_length);
-  rec_cache_size=info->cache_records*info->reclength;
-  info->rec_cache_size=info->cache_records*info->ref_length;
+  info->error_offset= info->table->s->reclength;
+  info->cache_records= (thd->variables.read_rnd_buff_size /
+                        (info->reclength+info->struct_length));
+  rec_cache_size= info->cache_records*info->reclength;
+  info->rec_cache_size= info->cache_records*info->ref_length;
 
   // We have to allocate one more byte to use uint3korr (see comments for it)
   if (info->cache_records <= 2 ||
@@ -346,7 +439,8 @@ static int init_rr_cache(READ_RECORD *info)
 					   MYF(0))))
     DBUG_RETURN(1);
 #ifdef HAVE_purify
-  bzero(info->cache,rec_cache_size);		// Avoid warnings in qsort
+  // Avoid warnings in qsort
+  bzero(info->cache,rec_cache_size+info->cache_records* info->struct_length+1);
 #endif
   DBUG_PRINT("info",("Allocated buffert for %d records",info->cache_records));
   info->read_positions=info->cache+rec_cache_size;
@@ -377,7 +471,8 @@ static int rr_from_cache(READ_RECORD *info)
       else
       {
 	error=0;
-	memcpy(info->record,info->cache_pos,(size_t) info->table->reclength);
+	memcpy(info->record,info->cache_pos,
+               (size_t) info->table->s->reclength);
       }
       info->cache_pos+=info->reclength;
       return ((int) error);
@@ -402,7 +497,8 @@ static int rr_from_cache(READ_RECORD *info)
       int3store(ref_position,(long) i);
       ref_position+=3;
     }
-    qsort(info->read_positions,length,info->struct_length,(qsort_cmp) rr_cmp);
+    my_qsort(info->read_positions, length, info->struct_length,
+             (qsort_cmp) rr_cmp);
 
     position=info->read_positions;
     for (i=0 ; i < length ; i++)
