@@ -307,8 +307,8 @@ bool String::copy_aligned(const char *str,uint32 arg_length, uint32 offset,
     return TRUE;
   
   /*
-    Note, this is only safe for little-endian UCS-2.
-    If we add big-endian UCS-2 sometimes, this code
+    Note, this is only safe for big-endian UCS-2.
+    If we add little-endian UCS-2 sometimes, this code
     will be more complicated. But it's OK for now.
   */
   bzero((char*) Ptr, offset);
@@ -617,27 +617,26 @@ skip:
 }
 
 /*
-** replace substring with string
-** If wrong parameter or not enough memory, do nothing
+  Replace substring with string
+  If wrong parameter or not enough memory, do nothing
 */
 
-
 bool String::replace(uint32 offset,uint32 arg_length,const String &to)
 {
   return replace(offset,arg_length,to.ptr(),to.length());
 }
 
 bool String::replace(uint32 offset,uint32 arg_length,
-                     const char *to,uint32 length)
+                     const char *to, uint32 to_length)
 {
-  long diff = (long) length-(long) arg_length;
+  long diff = (long) to_length-(long) arg_length;
   if (offset+arg_length <= str_length)
   {
     if (diff < 0)
     {
-      if (length)
-	memcpy(Ptr+offset,to,length);
-      bmove(Ptr+offset+length,Ptr+offset+arg_length,
+      if (to_length)
+	memcpy(Ptr+offset,to,to_length);
+      bmove(Ptr+offset+to_length,Ptr+offset+arg_length,
 	    str_length-offset-arg_length);
     }
     else
@@ -649,8 +648,8 @@ bool String::replace(uint32 offset,uint32 arg_length,
 	bmove_upp(Ptr+str_length+diff,Ptr+str_length,
 		  str_length-offset-arg_length);
       }
-      if (length)
-	memcpy(Ptr+offset,to,length);
+      if (to_length)
+	memcpy(Ptr+offset,to,to_length);
     }
     str_length+=(uint32) diff;
   }
@@ -859,7 +858,7 @@ outp:
   with optional left padding (for binary -> UCS2 conversion)
   
   SYNOPSIS
-    well_formed_copy_nhars()
+    well_formed_copy_nchars()
     to			     Store result here
     to_length                Maxinum length of "to" string
     to_cs		     Character set of "to" string
@@ -998,7 +997,10 @@ outp:
         goto outp;
       }
       else
+      {
+        from= from_prev;
         break;
+      }
     }
     *from_end_pos= from;
     res= to - to_start;
