@@ -47,14 +47,13 @@ char *my_defaults_extra_file=0;
 
 /* Which directories are searched for options (and in which order) */
 
-#define MAX_DEFAULT_DIRS 7
-const char *default_directories[MAX_DEFAULT_DIRS + 1];
+#define MAX_DEFAULT_DIRS 6
+#define DEFAULT_DIRS_SIZE (MAX_DEFAULT_DIRS + 1)  /* Terminate with NULL */
+static const char **default_directories = NULL;
 
 #ifdef __WIN__
 static const char *f_extensions[]= { ".ini", ".cnf", 0 };
 #define NEWLINE "\r\n"
-static char system_dir[FN_REFLEN], shared_system_dir[FN_REFLEN],
-            config_dir[FN_REFLEN];
 #else
 static const char *f_extensions[]= { ".cnf", 0 };
 #define NEWLINE "\n"
@@ -83,7 +82,37 @@ static int search_default_file_with_ext(Process_option_func func,
                                         void *func_ctx,
 					const char *dir, const char *ext,
 					const char *config_file, int recursion_level);
-static void init_default_directories();
+
+
+/**
+  Create the list of default directories.
+
+  @param alloc  MEM_ROOT where the list of directories is stored
+
+  @details
+  The directories searched, in order, are:
+  - Windows:     GetSystemWindowsDirectory()
+  - Windows:     GetWindowsDirectory()
+  - Windows:     C:/
+  - Windows:     Directory above where the executable is located
+  - Netware:     sys:/etc/
+  - Unix & OS/2: /etc/
+  - Unix:        --sysconfdir=<path> (compile-time option)
+  - OS/2:        getenv(ETC)
+  - ALL:         getenv(DEFAULT_HOME_ENV)
+  - ALL:         --defaults-extra-file=<path> (run-time option)
+  - Unix:        ~/
+
+  On all systems, if a directory is already in the list, it will be moved
+  to the end of the list.  This avoids reading defaults files multiple times,
+  while ensuring the correct precedence.
+
+  @retval NULL  Failure (out of memory, probably)
+  @retval other Pointer to NULL-terminated array of default directories
+*/
+
+static const char **init_default_directories(MEM_ROOT *alloc);
+
 
 static char *remove_end_comment(char *ptr);
 
@@ -371,8 +400,9 @@ int load_defaults(const char *conf_file, const char **groups,
   struct handle_option_ctx ctx;
   DBUG_ENTER("load_defaults");
 
-  init_default_directories();
   init_alloc_root(&alloc,512,0);
+  if ((default_directories= init_default_directories(&alloc)) == NULL)
+    goto err;
   /*
     Check if the user doesn't want any default option processing
     --no-defaults is always the first option
@@ -849,34 +879,49 @@ void my_print_default_files(const char *conf_file)
   my_bool have_ext= fn_ext(conf_file)[0] != 0;
   const char **exts_to_use= have_ext ? empty_list : f_extensions;
   char name[FN_REFLEN], **ext;
-  const char **dirs;
 
-  init_default_directories();
   puts("\nDefault options are read from the following files in the given order:");
 
   if (dirname_length(conf_file))
     fputs(conf_file,stdout);
   else
   {
-    for (dirs=default_directories ; *dirs; dirs++)
+    /*
+      If default_directories is already initialized, use it.  Otherwise,
+      use a private MEM_ROOT.
+    */
+    const char **dirs = default_directories;
+    MEM_ROOT alloc;
+    init_alloc_root(&alloc,512,0);
+
+    if (!dirs && (dirs= init_default_directories(&alloc)) == NULL)
     {
-      for (ext= (char**) exts_to_use; *ext; ext++)
+      fputs("Internal error initializing default directories list", stdout);
+    }
+    else
+    {
+      for ( ; *dirs; dirs++)
       {
-	const char *pos;
-	char *end;
-	if (**dirs)
-	  pos= *dirs;
-	else if (my_defaults_extra_file)
-	  pos= my_defaults_extra_file;
-	else
-	  continue;
-	end= convert_dirname(name, pos, NullS);
-	if (name[0] == FN_HOMELIB)	/* Add . to filenames in home */
-	  *end++='.';
-	strxmov(end, conf_file, *ext, " ", NullS);
-	fputs(name,stdout);
+        for (ext= (char**) exts_to_use; *ext; ext++)
+        {
+          const char *pos;
+          char *end;
+          if (**dirs)
+            pos= *dirs;
+          else if (my_defaults_extra_file)
+            pos= my_defaults_extra_file;
+          else
+            continue;
+          end= convert_dirname(name, pos, NullS);
+          if (name[0] == FN_HOMELIB)	/* Add . to filenames in home */
+            *end++= '.';
+          strxmov(end, conf_file, *ext, " ", NullS);
+          fputs(name, stdout);
+        }
       }
     }
+
+    free_root(&alloc, MYF(0));
   }
   puts("");
 }
@@ -913,6 +958,24 @@ void print_defaults(const char *conf_file, const char **groups)
 #include <help_end.h>
 
 
+static int add_directory(MEM_ROOT *alloc, const char *dir, const char **dirs)
+{
+  char buf[FN_REFLEN];
+  uint len;
+  char *p;
+  my_bool err __attribute__((unused));
+
+  len= normalize_dirname(buf, dir);
+  if (!(p= strmake_root(alloc, buf, len)))
+    return 1;  /* Failure */
+  /* Should never fail if DEFAULT_DIRS_SIZE is correct size */
+  err= array_append_string_unique(p, dirs, DEFAULT_DIRS_SIZE);
+  DBUG_ASSERT(err == FALSE);
+
+  return 0;
+}
+
+
 #ifdef __WIN__
 /*
   This wrapper for GetSystemWindowsDirectory() will dynamically bind to the
@@ -947,108 +1010,95 @@ static uint my_get_system_windows_directory(char *buffer, uint size)
   }
   return count;
 }
-#endif
 
 
-/*
-  Create the list of default directories.
-
-  On Microsoft Windows, this is:
-    1. C:/
-    2. GetWindowsDirectory()
-    3. GetSystemWindowsDirectory()
-    4. getenv(DEFAULT_HOME_ENV)
-    5. Directory above where the executable is located
-    6. ""
-    7. --sysconfdir=<path>
-
-  On Novell NetWare, this is:
-    1. sys:/etc/
-    2. getenv(DEFAULT_HOME_ENV)
-    3. ""
-    4. --sysconfdir=<path>
-
-  On OS/2, this is:
-    1. getenv(ETC)
-    2. /etc/
-    3. getenv(DEFAULT_HOME_ENV)
-    4. ""
-    5. "~/"
-    6. --sysconfdir=<path>
-
-  Everywhere else, this is:
-    1. /etc/
-    2. getenv(DEFAULT_HOME_ENV)
-    3. ""
-    4. "~/"
-    5. --sysconfdir=<path>
-
- */
-
-static void init_default_directories()
+static const char *my_get_module_parent(char *buf, size_t size)
 {
-  const char *env, **ptr= default_directories;
+  char *last= NULL;
+  char *end;
+  if (!GetModuleFileName(NULL, buf, (DWORD) size))
+    return NULL;
+  end= strend(buf);
+
+  /*
+    Look for the second-to-last \ in the filename, but hang on
+    to a pointer after the last \ in case we're in the root of
+    a drive.
+  */
+  for ( ; end > buf; end--)
+  {
+    if (*end == FN_LIBCHAR)
+    {
+      if (last)
+      {
+        /* Keep the last '\' as this works both with D:\ and a directory */
+        end[1]= 0;
+        break;
+      }
+      last= end;
+    }
+  }
+
+  return buf;
+}
+#endif /* __WIN__ */
+
+
+static const char **init_default_directories(MEM_ROOT *alloc)
+{
+  const char **dirs;
+  char *env;
+  int errors= 0;
+
+  dirs= (const char **)alloc_root(alloc, DEFAULT_DIRS_SIZE * sizeof(char *));
+  if (dirs == NULL)
+    return NULL;
+  bzero((char *) dirs, DEFAULT_DIRS_SIZE * sizeof(char *));
 
 #ifdef __WIN__
-  *ptr++= "C:/";
 
-  if (GetWindowsDirectory(system_dir,sizeof(system_dir)))
-    *ptr++= (char*)&system_dir;
-  if (my_get_system_windows_directory(shared_system_dir,
-                                      sizeof(shared_system_dir)) &&
-      strcmp(system_dir, shared_system_dir))
-    *ptr++= (char *)&shared_system_dir;
+  {
+    char fname_buffer[FN_REFLEN];
+    if (my_get_system_windows_directory(fname_buffer, sizeof(fname_buffer)))
+      errors += add_directory(alloc, fname_buffer, dirs);
+
+    if (GetWindowsDirectory(fname_buffer, sizeof(fname_buffer)))
+      errors += add_directory(alloc, fname_buffer, dirs);
+
+    errors += add_directory(alloc, "C:/", dirs);
+
+    if (my_get_module_parent(fname_buffer, sizeof(fname_buffer)) != NULL)
+      errors += add_directory(alloc, fname_buffer, dirs);
+  }
 
 #elif defined(__NETWARE__)
-  *ptr++= "sys:/etc/";
+
+  errors += add_directory(alloc, "sys:/etc/", dirs);
+
 #else
+
+  errors += add_directory(alloc, "/etc/", dirs);
+
 #if defined(__EMX__) || defined(OS2)
   if ((env= getenv("ETC")))
-    *ptr++= env;
-#endif
-  *ptr++= "/etc/";
-#endif
-  if ((env= getenv(STRINGIFY_ARG(DEFAULT_HOME_ENV))))
-    *ptr++= env;
-  *ptr++= "";			/* Place for defaults_extra_file */
-#if !defined(__WIN__) && !defined(__NETWARE__)
-  *ptr++= "~/";;
-#elif defined(__WIN__)
-  if (GetModuleFileName(NULL, config_dir, sizeof(config_dir)))
-  {
-    char *last= NULL, *end= strend(config_dir);
-    /*
-      Look for the second-to-last \ in the filename, but hang on
-      to a pointer after the last \ in case we're in the root of
-      a drive.
-    */
-    for ( ; end > config_dir; end--)
-    {
-      if (*end == FN_LIBCHAR)
-      {
-        if (last)
-        {
-          if (end != config_dir)
-          {
-            /* Keep the last '\' as this works both with D:\ and a directory */
-            end[1]= 0;
-          }
-          else
-          {
-            /* No parent directory (strange). Use current dir + '\' */
-            last[1]= 0;
-          }
-          break;
-        }
-        last= end;
-      }
-    }
-    *ptr++= (char *)&config_dir;
-  }
-#endif
-#ifdef DEFAULT_SYSCONFDIR
+    errors += add_directory(alloc, env, dirs);
+#elif defined(DEFAULT_SYSCONFDIR)
   if (DEFAULT_SYSCONFDIR != "")
-    *ptr++= DEFAULT_SYSCONFDIR;
+    errors += add_directory(alloc, DEFAULT_SYSCONFDIR, dirs);
+#endif /* __EMX__ || __OS2__ */
+
 #endif
-  *ptr= 0;			/* end marker */
+
+  if ((env= getenv(STRINGIFY_ARG(DEFAULT_HOME_ENV))))
+    errors += add_directory(alloc, env, dirs);
+
+  /* Placeholder for --defaults-extra-file=<path> */
+  errors += add_directory(alloc, "", dirs);
+
+#if !defined(__WIN__) && !defined(__NETWARE__) && \
+    !defined(__EMX__) && !defined(OS2)
+  errors += add_directory(alloc, "~/", dirs);
+#endif
+
+  return (errors > 0 ? NULL : dirs);
 }
