@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -75,7 +74,7 @@ void init_glob_errs()
   EE(EE_STAT)		= "Can't get stat of '%s' (Errcode: %d)";
   EE(EE_CANT_CHSIZE)	= "Can't change size of file (Errcode: %d)";
   EE(EE_CANT_OPEN_STREAM)= "Can't open stream from handle (Errcode: %d)";
-  EE(EE_GETWD)		= "Can't get working dirctory (Errcode: %d)";
+  EE(EE_GETWD)		= "Can't get working directory (Errcode: %d)";
   EE(EE_SETWD)		= "Can't change dir to '%s' (Errcode: %d)";
   EE(EE_LINK_WARNING)	= "Warning: '%s' had %d links";
   EE(EE_OPEN_WARNING)	= "%d files and %d streams is left open\n";
