@@ -540,13 +540,13 @@ extern const GlobalSignalNumber NO_OF_SIGNAL_NAMES;
 #define GSN_ABORT_ALL_REF               446
 #define GSN_ABORT_ALL_CONF              447
 
-#define GSN_STATISTICS_REQ              448
+/* 448 unused - formerly GSN_STATISTICS_REQ */
 #define GSN_STOP_ORD                    449
 #define GSN_TAMPER_ORD                  450
-#define GSN_SET_VAR_REQ                 451
-#define GSN_SET_VAR_CONF                452
-#define GSN_SET_VAR_REF                 453
-#define GSN_STATISTICS_CONF             454
+/* 451 unused - formerly GSN_SET_VAR_REQ  */
+/* 452 unused - formerly GSN_SET_VAR_CONF */
+/* 453 unused - formerly GSN_SET_VAR_REF  */
+/* 454 unused - formerly GSN_STATISTICS_CONF */
 
 #define GSN_START_ORD                   455
 /* 457 unused */
