@@ -816,8 +816,9 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
  InvalidSignal:
 #ifdef VM_TRACE
   ndbout_c("Ndbif: Error Ndb::handleReceivedSignal "
-	   "(GSN=%d, theImpl->theWaiter.m_state=%d)"
+	   "(tFirstDataPtr=%p, GSN=%d, theImpl->theWaiter.m_state=%d)"
 	   " sender = (Block: %d Node: %d)",
+           tFirstDataPtr,
 	   tSignalNumber,
 	   tWaitState,
 	   refToBlock(aSignal->theSendersBlockRef),
