@@ -102,11 +102,41 @@ case $system in
     ;;
 esac
 
+# This is needed to prefer GNU tar over platform tar because that can't
+# always handle long filenames
+
+PATH_DIRS=`echo $PATH | \
+    sed -e 's/^:/. /' -e 's/:$/ ./' -e 's/::/ . /g' -e 's/:/ /g' `
+
+which_1 ()
+{
+  for cmd
+  do
+    for d in $PATH_DIRS
+    do
+      for file in $d/$cmd
+      do
+	if [ -x $file -a ! -d $file ] ; then
+	  echo $file
+	  exit 0
+	fi
+      done
+    done
+  done
+  exit 1
+}
+
+tar=`which_1 gnutar gtar`
+if [ "$?" = "1" -o x"$tar" = x"" ] ; then
+  tar=tar
+fi
+
 
 mkdir $BASE $BASE/bin $BASE/docs \
  $BASE/include $BASE/lib $BASE/support-files $BASE/share $BASE/scripts \
  $BASE/mysql-test $BASE/mysql-test/t  $BASE/mysql-test/r \
- $BASE/mysql-test/include $BASE/mysql-test/std_data $BASE/mysql-test/lib
+ $BASE/mysql-test/include $BASE/mysql-test/std_data $BASE/mysql-test/lib \
+ $BASE/mysql-test/suite
 
 if [ $BASE_SYSTEM != "netware" ] ; then
  mkdir $BASE/share/mysql $BASE/tests $BASE/sql-bench $BASE/man \
@@ -115,7 +145,12 @@ if [ $BASE_SYSTEM != "netware" ] ; then
  chmod o-rwx $BASE/data $BASE/data/*
 fi
 
-# Copy files if they exists, warn for those that don't
+# Copy files if they exists, warn for those that don't.
+# Note that when listing files to copy, we might list the file name
+# twice, once in the directory location where it is built, and a
+# second time in the ".libs" location. In the case the first one
+# is a wrapper script, the second one will overwrite it with the
+# binary file.
 copyfileto()
 {
   destdir=$1
@@ -143,7 +178,8 @@ BIN_FILES="extra/comp_err$BS extra/replace$BS extra/perror$BS \
   extra/resolve_stack_dump$BS extra/mysql_waitpid$BS \
   myisam/myisamchk$BS myisam/myisampack$BS myisam/myisamlog$BS \
   myisam/myisam_ftdump$BS \
-  sql/mysqld$BS sql/mysql_tzinfo_to_sql$BS \
+  sql/mysqld$BS sql/mysqld-debug$BS \
+  sql/mysql_tzinfo_to_sql$BS \
   server-tools/instance-manager/mysqlmanager$BS \
   client/mysql$BS client/mysqlshow$BS client/mysqladmin$BS \
   client/mysqldump$BS client/mysqlimport$BS \
@@ -164,6 +200,7 @@ if [ $BASE_SYSTEM = "netware" ] ; then
 # For all other platforms:
 else
   BIN_FILES="$BIN_FILES \
+    server-tools/instance-manager/.libs/mysqlmanager \
     client/mysqltestmanagerc \
     client/mysqltestmanager-pwgen tools/mysqltestmanager \
     client/.libs/mysql client/.libs/mysqlshow client/.libs/mysqladmin \
@@ -220,7 +257,7 @@ if [ $BASE_SYSTEM = "netware" ] ; then
     rm -f $BASE/lib/*.la
 fi
 
-copyfileto $BASE/include config.h include/*
+copyfileto $BASE/include include/*
 
 rm -f $BASE/include/Makefile* $BASE/include/*.in $BASE/include/config-win.h
 if [ $BASE_SYSTEM != "netware" ] ; then
@@ -247,33 +284,35 @@ rm -f $MYSQL_SHARE/Makefile* $MYSQL_SHARE/*/*.OLD
 copyfileto $BASE/mysql-test \
          mysql-test/mysql-test-run mysql-test/install_test_db \
          mysql-test/mysql-test-run.pl mysql-test/README \
-	 mysql-test/valgrind.supp \
+         mysql-test/mysql-stress-test.pl \
+         mysql-test/valgrind.supp \
          netware/mysql_test_run.nlm netware/install_test_db.ncf
 
 $CP mysql-test/lib/*.pl  $BASE/mysql-test/lib
-$CP mysql-test/lib/*.sql $BASE/mysql-test/lib
 $CP mysql-test/t/*.def $BASE/mysql-test/t
 $CP mysql-test/include/*.inc $BASE/mysql-test/include
 $CP mysql-test/include/*.test $BASE/mysql-test/include
 $CP mysql-test/t/*.def $BASE/mysql-test/t
-$CP mysql-test/std_data/*.dat mysql-test/std_data/*.frm \
-    mysql-test/std_data/*.MYD mysql-test/std_data/*.MYI \
-    mysql-test/std_data/*.pem mysql-test/std_data/Moscow_leap \
-    mysql-test/std_data/des_key_file mysql-test/std_data/*.*001 \
-    mysql-test/std_data/*.cnf mysql-test/std_data/*.MY* \
-    $BASE/mysql-test/std_data
 $CP mysql-test/t/*.test mysql-test/t/*.imtest \
     mysql-test/t/*.disabled mysql-test/t/*.opt \
     mysql-test/t/*.slave-mi mysql-test/t/*.sh mysql-test/t/*.sql $BASE/mysql-test/t
 $CP mysql-test/r/*.result mysql-test/r/*.require \
     $BASE/mysql-test/r
 
+# Copy the additional suites and data "as is", they are in flux
+$tar cf - mysql-test/suite    | ( cd $BASE ; $tar xf - )
+$tar cf - mysql-test/std_data | ( cd $BASE ; $tar xf - )
+# Clean up if we did this from a bk tree
+if [ -d mysql-test/SCCS ] ; then
+  find $BASE/mysql-test -name SCCS -print | xargs rm -rf
+fi
+
 if [ $BASE_SYSTEM != "netware" ] ; then
   chmod a+x $BASE/bin/*
   copyfileto $BASE/bin scripts/*
   $BASE/bin/replace \@localstatedir\@ ./data \@bindir\@ ./bin \@scriptdir\@ \
       ./bin \@libexecdir\@ ./bin \@sbindir\@ ./bin \@prefix\@ . \@HOSTNAME\@ \
-      @HOSTNAME@ \@pkgdatadir\@ ./support-files \
+      @HOSTNAME@ \@pkgdatadir\@ ./share \
       < scripts/mysql_install_db.sh > $BASE/scripts/mysql_install_db
   $BASE/bin/replace \@prefix\@ /usr/local/mysql \@bindir\@ ./bin \
       \@sbindir\@ ./bin \@libexecdir\@ ./bin \
@@ -282,8 +321,8 @@ if [ $BASE_SYSTEM != "netware" ] ; then
       < support-files/mysql.server.sh > $BASE/support-files/mysql.server
   $BASE/bin/replace /my/gnu/bin/hostname /bin/hostname -- $BASE/bin/mysqld_safe
   mv $BASE/support-files/binary-configure $BASE/configure
-  chmod a+x $BASE/bin/* $BASE/scripts/* $BASE/support-files/mysql-* \
-      $BASE/support-files/mysql.server $BASE/configure
+  chmod a+x $BASE/bin/* $BASE/scripts/* $BASE/support-files/mysql-log-rotate \
+      $BASE/support-files/*.server $BASE/configure
   $CP -r sql-bench/* $BASE/sql-bench
   rm -f $BASE/sql-bench/*.sh $BASE/sql-bench/Makefile* $BASE/lib/*.la
   rm -f $BASE/bin/*.sql
@@ -298,12 +337,6 @@ rm -f $BASE/bin/Makefile* $BASE/bin/*.in $BASE/bin/*.sh \
 # Copy system dependent files
 #
 if [ $BASE_SYSTEM = "netware" ] ; then
-  echo "CREATE DATABASE mysql;" > $BASE/bin/init_db.sql
-  echo "CREATE DATABASE test;" >> $BASE/bin/init_db.sql
-  sh ./scripts/mysql_create_system_tables.sh real "" "%" 0 \
-      >> $BASE/bin/init_db.sql
-  sh ./scripts/mysql_create_system_tables.sh test "" "%" 0 \
-      > $BASE/bin/test_db.sql
   ./scripts/fill_help_tables < ./Docs/manual.texi >> ./netware/init_db.sql
 fi
 
@@ -361,11 +394,13 @@ BASE=$BASE2
 #
 
 if [ x"@GXX@" = x"yes" ] ; then
-  gcclib=`@CC@ @CFLAGS@ --print-libgcc-file`
-  if [ $? -ne 0 ] ; then
-    echo "Warning: Couldn't find libgcc.a!"
-  else
+  gcclib=`@CC@ @CFLAGS@ --print-libgcc-file 2>/dev/null` || true
+  if [ -z "$gcclib" ] ; then
+    echo "Warning: Compiler doesn't tell libgcc.a!"
+  elif [ -f "$gcclib" ] ; then
     $CP $gcclib $BASE/lib/libmygcc.a
+  else
+    echo "Warning: Compiler result '$gcclib' not found / no file!"
   fi
 fi
 
@@ -374,41 +409,12 @@ if [ x$DEBUG = x1 ] ; then
  exit
 fi
 
-# This is needed to prefere gnu tar instead of tar because tar can't
-# always handle long filenames
-
-PATH_DIRS=`echo $PATH | \
-    sed -e 's/^:/. /' -e 's/:$/ ./' -e 's/::/ . /g' -e 's/:/ /g' `
-
-which_1 ()
-{
-  for cmd
-  do
-    for d in $PATH_DIRS
-    do
-      for file in $d/$cmd
-      do
-	if [ -x $file -a ! -d $file ] ; then
-	  echo $file
-	  exit 0
-	fi
-      done
-    done
-  done
-  exit 1
-}
-
 if [ $BASE_SYSTEM != "netware" ] ; then
 
   #
   # Create the result tar file
   #
 
-  tar=`which_1 gnutar gtar`
-  if [ "$?" = "1" -o x"$tar" = x"" ] ; then
-    tar=tar
-  fi
-
   echo "Using $tar to create archive"
 
   OPT=cvf
