@@ -524,7 +524,7 @@ int send_variant_2_list(MEM_ROOT *mem_root, Protocol *protocol,
   List_iterator<String> it(*names);
   for (pos= pointers; pos!=end; (*pos++= it++));
 
-  qsort(pointers,names->elements,sizeof(String*),string_ptr_cmp);
+  my_qsort(pointers,names->elements,sizeof(String*),string_ptr_cmp);
 
   for (pos= pointers; pos!=end; pos++)
   {
@@ -656,6 +656,8 @@ bool mysqld_help(THD *thd, const char *mask)
     Init tables and fields to be usable from items
     tables do not contain VIEWs => we can pass 0 as conds
   */
+  thd->lex->select_lex.context.table_list= 
+    thd->lex->select_lex.context.first_name_resolution_table= &tables[0];
   setup_tables(thd, &thd->lex->select_lex.context,
                &thd->lex->select_lex.top_join_list,
                tables, 0, &leaves, FALSE);
