@@ -37,7 +37,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <violite.h>
-#include <md5.h>
+#include "my_md5.h"
 #include <signal.h>
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
@@ -1312,7 +1312,7 @@ static void usage()
 
 static my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
-	       char *argument)
+	       char *argument __attribute__((unused)))
 {
   switch (optid) {
   case '#':
